#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Contributors to the Ansible project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: route53_key_signing_key
short_description: Manages a key-signing key (KSK)
version_added: 9.2.0
description:
    - Creates a new key-signing key (KSK) associated with a hosted zone.
      You can only have two KSKs per hosted zone.
    - When O(state=absent), it deactivates and deletes a key-signing key (KSK).
    - Activates a key-signing key (KSK) so that it can be used for signing by DNSSEC.
    - Deactivates a key-signing key (KSK) so that it will not be used for signing by DNSSEC.
options:
    state:
        description:
            - Whether or not the zone should exist.
        default: present
        choices: [ "present", "absent" ]
        type: str
    caller_reference:
        description:
            - A unique string that identifies the request.
            - Required when O(state=present).
        type: str
    hosted_zone_id:
        description:
            - The unique string (ID) used to identify a hosted zone.
        type: str
        required: true
        aliases: ["zone_id"]
    key_management_service_arn:
        description:
            - The Amazon resource name (ARN) for a customer managed key in Key Management Service (KMS).
            - Required when O(state=present).
        type: str
        aliases: ["kms_arn"]
    name:
        description:
            - A string used to identify a key-signing key (KSK).
        type: str
        required: true
    status:
        description:
            - A string specifying the initial status of the key-signing key (KSK).
            - When O(state=presnent), you can set the value to V(ACTIVE) or V(INACTIVE).
        type: str
        default: "ACTIVE"
        choices: ["ACTIVE", "INACTIVE"]
    wait:
        description:
            - Wait until the changes have been replicated.
        type: bool
        default: false
    wait_timeout:
        description:
        - How long to wait for the changes to be replicated, in seconds.
        default: 300
        type: int
extends_documentation_fragment:
    - amazon.aws.common.modules
    - amazon.aws.region.modules
    - amazon.aws.boto3
author:
    - Alina Buzachis (@alinabuzachis)
"""

EXAMPLES = r"""
- name: Create a Key Signing Key Request
  amazon.aws.route53_ksk:
    name: "{{ resource_prefix }}-ksk"
    hosted_zone_id: "ZZZ1111112222"
    key_management_service_arn: "arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"
    caller_reference: "arn:aws:iam::123456789012:user/SomeUser"
    status: "INACTIVE"
    state: present

- name: Activate a Key Signing Key Request
  amazon.aws.route53_ksk:
    name: "{{ resource_prefix }}-ksk"
    hosted_zone_id: "ZZZ1111112222"
    key_management_service_arn: "arn:aws:kms:us-west-2:111122223333:key/1234abcd-12ab-34cd-56ef-1234567890ab"
    caller_reference: "arn:aws:iam::123456789012:user/SomeUser"
    status: "ACTIVE"
    state: present

- name: Delete a Key Signing Key Request and deactivate it
  amazon.aws.route53_ksk:
    name: "{{ resource_prefix }}-ksk"
    hosted_zone_id: "ZZZ1111112222"
    state: absent
"""

RETURN = r"""
change_info:
    description: A dictionary that describes change information about changes made to the hosted zone.
    returned: when a Key Signing Key is created or it exists and is updated/deleted
    type: dict
    contains:
        id:
            description: Change ID.
            type: str
        status:
            description: The current state of the request.
            type: str
        submitted_at:
            description: The date and time that the change request was submitted in ISO 8601 format and Coordinated Universal Time (UTC).
            type: str
        comment:
            description: A comment you can provide.
            type: str
    sample: {
        "id": "/change/C090307813XORZJ5J3U4",
        "status": "PENDING",
        "submitted_at": "2024-12-04T15:15:36.743000+00:00"
    }
location:
    description: The unique URL representing the new key-signing key (KSK).
    returned: when only a new Key Signing Key is created
    type: str
    sample: "https://route53.amazonaws.com/2013-04-01/keysigningkey/xxx/ansible-test-ksk"
key_signing_key:
    description: The key-signing key (KSK) that the request creates.
    returned: always
    type: dict
    contains:
        name:
            description: A string used to identify a key-signing key (KSK).
            type: str
        kms_arn:
            description: The Amazon resource name (ARN) used to identify the customer managed key in Key Management Service (KMS).
            type: str
        flag:
            description: An integer that specifies how the key is used.
            type: int
        signing_algorithm_mnemonic:
            description: A string used to represent the signing algorithm.
            type: str
        signing_algorithm_type:
            description: An integer used to represent the signing algorithm.
            type: int
        digest_algorithm_mnemonic:
            description: A string used to represent the delegation signer digest algorithm.
            type: str
        digest_algorithm_type:
            description: An integer used to represent the delegation signer digest algorithm.
            type: int
        key_tag:
            description: An integer used to identify the DNSSEC record for the domain name.
            type: int
        digest_value:
            description: A cryptographic digest of a DNSKEY resource record (RR).
            type: str
        public_key:
            description: The public key, represented as a Base64 encoding.
            type: str
        ds_record:
            description: A string that represents a delegation signer (DS) record.
            type: str
        dnskey_record:
            description: A string that represents a DNSKEY record.
            type: str
        status:
            description: A string that represents the current key-signing key (KSK) status.
            type: str
        status_message:
            description: The status message provided for ACTION_NEEDED or INTERNAL_FAILURE statuses.
            type: str
        created_date:
            description: The date when the key-signing key (KSK) was created.
            type: str
        last_modified_date:
            description: The last time that the key-signing key (KSK) was changed.
            type: str
    sample: {
        "created_date": "2024-12-04T15:15:36.715000+00:00",
        "digest_algorithm_mnemonic": "SHA-256",
        "digest_algorithm_type": 2,
        "digest_value": "xxx",
        "dnskey_record": "xxx",
        "ds_record": "xxx",
        "flag": 257,
        "key_tag": 18948,
        "kms_arn": "arn:aws:kms:us-east-1:xxx:key/xxx",
        "last_modified_date": "2024-12-04T15:15:36.715000+00:00",
        "name": "ansible-test-44230979--ksk",
        "public_key": "xxxx",
        "signing_algorithm_mnemonic": "ECDSAP256SHA256",
        "signing_algorithm_type": 13,
        "status": "INACTIVE"
    }
"""


try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from typing import Any
from typing import Dict
from typing import Tuple

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.exceptions import AnsibleAWSError
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.waiters import get_waiter


def deactivate_key_signing_key(client, hosted_zone_id: str, name: str) -> Dict[str, Any]:
    return client.deactivate_key_signing_key(HostedZoneId=hosted_zone_id, Name=name)


def activate_key_signing_key(client, hosted_zone_id: str, name: str) -> Dict[str, Any]:
    return client.activate_key_signing_key(HostedZoneId=hosted_zone_id, Name=name)


def get_change(client, change_id: str) -> Dict[str, Any]:
    return client.get_change(Id=change_id)


def get_dnssec(client, hosted_zone_id: str) -> Dict[str, Any]:
    return client.get_dnssec(HostedZoneId=hosted_zone_id)


def find_ksk(client, module: AnsibleAWSModule) -> Dict[str, Any]:
    hosted_zone_dnssec = get_dnssec(client, module.params.get("hosted_zone_id"))
    if hosted_zone_dnssec["KeySigningKeys"] != []:
        for ksk in hosted_zone_dnssec["KeySigningKeys"]:
            if ksk["Name"] == module.params.get("name"):
                return ksk
    return {}


def wait(client, module: AnsibleAWSModule, change_id: str) -> None:
    try:
        waiter = get_waiter(client, "resource_record_sets_changed")
        waiter.wait(
            Id=change_id,
            WaiterConfig=dict(
                Delay=5,
                MaxAttempts=module.params.get("wait_timeout") // 5,
            ),
        )
    except botocore.exceptions.WaiterError as e:
        module.fail_json_aws(e, msg="Timeout waiting for changes to be applied")


def create_or_update(client, module: AnsibleAWSModule, ksk: Dict[str, Any]) -> Tuple[bool, Dict[str, Any]]:
    changed: bool = False
    zone_id = module.params.get("hosted_zone_id")
    name = module.params.get("name")
    status = module.params.get("status")
    response = {}

    if ksk:
        if ksk["Status"] != status:
            changed = True

            if module.check_mode:
                ksk["Status"] = status
                module.exit_json(
                    changed=changed,
                    key_signing_key=camel_dict_to_snake_dict(ksk),
                    msg=f"Would have updated the Key Signing Key status to {status} if not in check_mode.",
                )

            if status == "ACTIVE":
                response.update(activate_key_signing_key(client, zone_id, name))
            elif status == "INACTIVE":
                response.update(deactivate_key_signing_key(client, zone_id, name))
    else:
        changed = True
        if module.check_mode:
            module.exit_json(changed=changed, msg="Would have created the Key Signing Key if not in check_mode.")

        response = client.create_key_signing_key(
            CallerReference=module.params.get("caller_reference"),
            KeyManagementServiceArn=module.params.get("key_management_service_arn"),
            HostedZoneId=zone_id,
            Name=name,
            Status=status,
        )

    return changed, response


def delete(client, module: AnsibleAWSModule, ksk: Dict[str, Any]) -> Tuple[bool, Dict[str, Any]]:
    changed: bool = False
    zone_id = module.params.get("hosted_zone_id")
    name = module.params.get("name")
    response = {}

    if ksk:
        changed = True
        if module.check_mode:
            module.exit_json(
                changed=changed,
                key_signing_key={},
                msg="Would have deleted the Key Signing Key if not in check_mode.",
            )

        result = deactivate_key_signing_key(client, zone_id, name)
        change_id = result["ChangeInfo"]["Id"]
        wait(client, module, change_id)

        response.update(client.delete_key_signing_key(HostedZoneId=zone_id, Name=name))

    return changed, response


def main() -> None:
    argument_spec = dict(
        caller_reference=dict(type="str"),
        hosted_zone_id=dict(type="str", aliases=["zone_id"], required=True),
        key_management_service_arn=dict(type="str", aliases=["kms_arn"], no_log=False),
        name=dict(type="str", required=True),
        status=dict(type="str", default="ACTIVE", choices=["ACTIVE", "INACTIVE"]),
        state=dict(default="present", choices=["present", "absent"]),
        wait=dict(type="bool", default=False),
        wait_timeout=dict(type="int", default=300),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[["state", "present", ["caller_reference", "key_management_service_arn"]]],
    )

    try:
        client = module.client("route53")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    changed = False
    state = module.params.get("state")

    try:
        ksk = find_ksk(client, module)
        if state == "present":
            changed, result = create_or_update(client, module, ksk)
        else:
            changed, result = delete(client, module, ksk)

        if module.params.get("wait") and result.get("ChangeInfo"):
            change_id = result["ChangeInfo"]["Id"]
            wait(client, module, change_id)
            result.update(get_change(client, change_id))

        # Get updated information about KSK
        result["KeySigningKey"] = find_ksk(client, module)

        if "ResponseMetadata" in result:
            del result["ResponseMetadata"]

    except AnsibleAWSError as e:
        module.fail_json_aws_error(e)

    module.exit_json(changed=changed, **camel_dict_to_snake_dict(result))


if __name__ == "__main__":
    main()



# Copyright (c) 2019-2020, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.docker.plugins.module_utils._socket_handler import (
    DockerSocketHandlerBase,
)

if t.TYPE_CHECKING:
    from ansible.utils.display import Display

    from ansible_collections.community.docker.plugins.module_utils._socket_helper import (
        SocketLike,
    )


class DockerSocketHandler(DockerSocketHandlerBase):
    def __init__(
        self, display: Display, sock: SocketLike, container: str | None = None
    ) -> None:
        super().__init__(sock, log=lambda msg: display.vvvv(msg, host=container))



#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
#
# Python SDK Reference: https://learn.microsoft.com/en-us/python/api/azure-mgmt-cdn/azure.mgmt.cdn.operations.afdorigingroupsoperations?view=azure-python
#

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_afdorigingroup
version_added: "3.4.0"
short_description: Manage an Azure Front Door OriginGroup to be used with Standard or Premium Frontdoor
description:
    - Create, update and delete an Azure Front Door (AFD) OriginGroup to be used by a Front Door Service Profile created using azure_rm_cdnprofile.
    - This differs from the Front Door classic service and only is intended to be used by the Standard or Premium service offering.

options:
    health_probe_settings:
        description:
            - Health probe settings to the origin that is used to determine the health of the origin.
        type: dict
        suboptions:
            probe_interval_in_seconds:
                description:
                    - The number of seconds between health probes.
                type: int
            probe_path:
                description:
                    - The path relative to the origin that is used to determine the health of the origin.
                type: str
            probe_protocol:
                description:
                    - Protocol to use for health probe.
                type: str
                choices:
                    - Http
                    - Https
                    - NotSet
            probe_request_type:
                description:
                    - The type of health probe request that is made.
                type: str
                choices:
                    - GET
                    - HEAD
                    - NotSet
    load_balancing_settings:
        description:
            - Load balancing settings for a backend pool.
        type: dict
        suboptions:
            additional_latency_in_milliseconds:
                description:
                    - The additional latency in milliseconds for probes to fall into the lowest latency bucket.
                type: int
            sample_size:
                description:
                    - The number of samples to consider for load balancing decisions.
                type: int
            successful_samples_required:
                description:
                    - The number of samples within the sample period that must succeed.
                type: int
    name:
        description:
            - Name of the AFD OriginGroup.
        required: true
        type: str
    profile_name:
        description:
            - Name of the AFD Profile where the OriginGroup will be added.
        required: true
        type: str
    resource_group:
        description:
            - Name of a resource group where the AFD OriginGroup exists or will be created.
        required: true
        type: str
    session_affinity_state:
        description:
            - Whether to allow session affinity on this host.
        type: str
        choices:
            - Enabled
            - Disabled
    state:
        description:
            - Assert the state of the AFD OriginGroup. Use C(present) to create or update an AFD OriginGroup and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Jarret Tooley (@jartoo)
'''

EXAMPLES = '''
- name: Create an AFD OriginGroup
  azure_rm_afdorigingroup:
    name: myOriginGroup
    profile_name: myProfile
    resource_group: myResourceGroup
    state: present
    tags:
      testing: testing

- name: Delete the AFD OriginGroup
  azure_rm_afdorigingroup:
    name: myOriginGroup
    profile_name: myProfile
    resource_group: myResourceGroup
    state: absent
'''
RETURN = '''
id:
    description:
        - ID of the AFD OriginGroup.
    returned: always
    type: str
    sample: "id: /subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourcegroups/myResourceGroup/providers/Microsoft.Cdn/profiles/myProf/origingroups/myOG"
'''
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.mgmt.cdn.models import AFDOriginGroup, LoadBalancingSettingsParameters, HealthProbeParameters
except ImportError as ec:
    # This is handled in azure_rm_common
    pass


def origingroup_to_dict(origingroup):
    return dict(
        additional_latency_in_milliseconds=origingroup.load_balancing_settings.additional_latency_in_milliseconds,
        deployment_status=origingroup.deployment_status,
        id=origingroup.id,
        name=origingroup.name,
        probe_interval_in_seconds=origingroup.health_probe_settings.probe_interval_in_seconds,
        probe_path=origingroup.health_probe_settings.probe_path,
        probe_protocol=origingroup.health_probe_settings.probe_protocol,
        probe_request_type=origingroup.health_probe_settings.probe_request_type,
        provisioning_state=origingroup.provisioning_state,
        sample_size=origingroup.load_balancing_settings.sample_size,
        session_affinity_state=origingroup.session_affinity_state,
        successful_samples_required=origingroup.load_balancing_settings.successful_samples_required,
        traffic_restoration_time_to_healed_or_new_endpoints_in_minutes=origingroup.traffic_restoration_time_to_healed_or_new_endpoints_in_minutes,
        type=origingroup.type
    )


class AzureRMOriginGroup(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            load_balancing_settings=dict(
                type='dict',
                options=dict(
                    additional_latency_in_milliseconds=dict(type='int'),
                    sample_size=dict(type='int'),
                    successful_samples_required=dict(type='int')
                )
            ),
            health_probe_settings=dict(
                type='dict',
                options=dict(
                    probe_path=dict(type='str'),
                    probe_request_type=dict(type='str', choices=['GET', 'HEAD', 'NotSet']),
                    probe_protocol=dict(type='str', choices=['Http', 'Https', 'NotSet']),
                    probe_interval_in_seconds=dict(type='int')
                )
            ),
            name=dict(
                type='str',
                required=True
            ),
            profile_name=dict(
                type='str',
                required=True
            ),
            resource_group=dict(
                type='str',
                required=True
            ),
            session_affinity_state=dict(
                type='str',
                choices=['Enabled', 'Disabled']
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            )
        )

        self.health_probe_settings = dict()
        self.health_probe_settings['probe_path'] = None
        self.health_probe_settings['probe_request_type'] = None
        self.health_probe_settings['probe_protocol'] = None
        self.health_probe_settings['probe_interval_in_seconds'] = None
        self.load_balancing_settings = dict()
        self.load_balancing_settings['additional_latency_in_milliseconds'] = None
        self.load_balancing_settings['sample_size'] = None
        self.load_balancing_settings['successful_samples_required'] = None
        self.session_affinity_state = None

        self.name = None
        self.profile_name = None
        self.resource_group = None
        self.state = None

        self.results = dict(changed=False)

        super(AzureRMOriginGroup, self).__init__(
            derived_arg_spec=self.module_arg_spec,
            supports_check_mode=True,
            supports_tags=False)

    def exec_module(self, **kwargs):
        """Main module execution method"""

        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        to_be_updated = False

        response = self.get_origingroup()

        if self.state == 'present':

            if not response:
                self.log("Need to create the OriginGroup")

                if not self.check_mode:
                    new_response = self.create_origingroup()
                    self.results['id'] = new_response['id']

                self.results['changed'] = True

            else:
                self.log('Results : {0}'.format(response))
                self.results['id'] = response['id']

                if response['probe_path'] != self.health_probe_settings['probe_path'] and self.health_probe_settings['probe_path']:
                    to_be_updated = True
                if response['probe_request_type'] != self.health_probe_settings['probe_request_type'] and self.health_probe_settings['probe_request_type']:
                    to_be_updated = True
                if response['probe_protocol'] != self.health_probe_settings['probe_protocol'] and self.health_probe_settings['probe_protocol']:
                    to_be_updated = True
                if response['probe_interval_in_seconds'] != self.health_probe_settings['probe_interval_in_seconds'] and \
                        self.health_probe_settings['probe_interval_in_seconds']:
                    to_be_updated = True
                if response['sample_size'] != self.load_balancing_settings['sample_size'] and self.load_balancing_settings['sample_size']:
                    to_be_updated = True
                if response['successful_samples_required'] != self.load_balancing_settings['successful_samples_required'] and \
                        self.load_balancing_settings['successful_samples_required']:
                    to_be_updated = True
                if response['additional_latency_in_milliseconds'] != self.load_balancing_settings['additional_latency_in_milliseconds'] and \
                        self.load_balancing_settings['additional_latency_in_milliseconds']:
                    to_be_updated = True
                if response['session_affinity_state'] != self.session_affinity_state and self.session_affinity_state:
                    to_be_updated = True

                if to_be_updated:
                    self.log("Need to update the AFD OriginGroup")
                    self.results['id'] = response['id']

                    if not self.check_mode:
                        new_response = self.update_origingroup()
                        self.results['id'] = new_response['id']

                    self.results['changed'] = True

        elif self.state == 'absent':
            if not response:
                self.log("AFD OriginGroup {0} does not exist.".format(self.name))
                self.results['id'] = ""
            else:
                self.log("Need to delete the OriginGroup")
                self.results['changed'] = True
                self.results['id'] = response['id']
                if not self.check_mode:
                    self.delete_origingroup()
                    self.log("Azure AFD OriginGroup deleted")

        return self.results

    def create_origingroup(self):
        '''
        Creates a Azure OriginGroup.

        :return: deserialized Azure OriginGroup instance state dictionary
        '''
        self.log("Creating the Azure OriginGroup instance {0}".format(self.name))

        loadbalancingsettings = LoadBalancingSettingsParameters(
            sample_size=self.load_balancing_settings['sample_size'],
            successful_samples_required=self.load_balancing_settings['successful_samples_required'],
            additional_latency_in_milliseconds=self.load_balancing_settings['additional_latency_in_milliseconds']
        )

        healthprobesettings = HealthProbeParameters(
            probe_path=self.health_probe_settings['probe_path'],
            probe_request_type=self.health_probe_settings['probe_request_type'],
            probe_protocol=self.health_probe_settings['probe_protocol'],
            probe_interval_in_seconds=self.health_probe_settings['probe_interval_in_seconds']
        )

        parameters = AFDOriginGroup(
            load_balancing_settings=loadbalancingsettings,
            health_probe_settings=healthprobesettings,
            session_affinity_state=self.session_affinity_state
        )

        try:
            poller = self.cdn_client.afd_origin_groups.begin_create(self.resource_group,
                                                                    self.profile_name,
                                                                    self.name,
                                                                    parameters)
            response = self.get_poller_result(poller)
            return origingroup_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to create Azure OriginGroup instance.')
            self.fail("Error Creating Azure OriginGroup instance: {0}".format(exc.message))

    def update_origingroup(self):
        '''
        Updates an Azure OriginGroup.

        :return: deserialized Azure OriginGroup instance state dictionary
        '''
        self.log("Updating the Azure OriginGroup instance {0}".format(self.name))

        loadbalancingsettings = LoadBalancingSettingsParameters(
            sample_size=self.load_balancing_settings['sample_size'],
            successful_samples_required=self.load_balancing_settings['successful_samples_required'],
            additional_latency_in_milliseconds=self.load_balancing_settings['additional_latency_in_milliseconds']
        )

        healthprobesettings = HealthProbeParameters(
            probe_path=self.health_probe_settings['probe_path'],
            probe_request_type=self.health_probe_settings['probe_request_type'],
            probe_protocol=self.health_probe_settings['probe_protocol'],
            probe_interval_in_seconds=self.health_probe_settings['probe_interval_in_seconds']
        )

        parameters = AFDOriginGroup(
            load_balancing_settings=loadbalancingsettings,
            health_probe_settings=healthprobesettings,
            session_affinity_state=self.session_affinity_state
        )

        try:
            poller = self.cdn_client.afd_origin_groups.begin_update(
                resource_group_name=self.resource_group,
                profile_name=self.profile_name,
                origin_group_name=self.name,
                origin_group_update_properties=parameters)
            response = self.get_poller_result(poller)
            return origingroup_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to update Azure OriginGroup instance.')
            self.fail("Error updating Azure OriginGroup instance: {0}".format(exc.message))

    def delete_origingroup(self):
        '''
        Deletes the specified Azure OriginGroup in the specified subscription and resource group.

        :return: True
        '''
        self.log("Deleting the OriginGroup {0}".format(self.name))
        try:
            poller = self.cdn_client.afd_origin_groups.begin_delete(self.resource_group, self.profile_name, self.name)
            self.get_poller_result(poller)
            return True
        except Exception as e:
            self.log('Error attempting to delete the OriginGroup.')
            self.fail("Error deleting the OriginGroup: {0}".format(e.message))
            return False

    def get_origingroup(self):
        '''
        Gets the properties of the specified OriginGroup.

        :return: deserialized OriginGroup state dictionary
        '''
        self.log(
            "Checking if the OriginGroup {0} is present".format(self.name))
        try:
            response = self.cdn_client.afd_origin_groups.get(self.resource_group, self.profile_name, self.name)
            self.log("Response : {0}".format(response))
            self.log("OriginGroup : {0} found".format(response.name))
            return origingroup_to_dict(response)
        except Exception as err:
            self.log('Did not find the OriginGroup.' + err.args[0])
            return False


def main():
    """Main execution"""
    AzureRMOriginGroup()


if __name__ == '__main__':
    main()



#!/usr/bin/python
# Copyright (c) 2017, Abhijeet Kasurde (akasurde@redhat.com)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_subca
author: Abhijeet Kasurde (@Akasurde)
short_description: Manage FreeIPA Lightweight Sub Certificate Authorities
description:
  - Add, modify, enable, disable and delete an IPA Lightweight Sub Certificate Authorities using IPA API.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  subca_name:
    description:
      - The Sub Certificate Authority name which needs to be managed.
    required: true
    aliases: ["name"]
    type: str
  subca_subject:
    description:
      - The Sub Certificate Authority's Subject, for example V(CN=SampleSubCA1,O=testrelm.test).
    required: true
    type: str
  subca_desc:
    description:
      - The Sub Certificate Authority's description.
    type: str
  state:
    description:
      - State to ensure.
      - States V(disable) and V(enable) are available for FreeIPA 4.4.2 version and onwards.
    default: present
    choices: ["absent", "disabled", "enabled", "present"]
    type: str
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure IPA Sub CA is present
  community.general.ipa_subca:
    ipa_host: spider.example.com
    ipa_pass: Passw0rd!
    state: present
    subca_name: AnsibleSubCA1
    subca_subject: 'CN=AnsibleSubCA1,O=example.com'
    subca_desc: Ansible Sub CA

- name: Ensure that IPA Sub CA is removed
  community.general.ipa_subca:
    ipa_host: spider.example.com
    ipa_pass: Passw0rd!
    state: absent
    subca_name: AnsibleSubCA1

- name: Ensure that IPA Sub CA is disabled
  community.general.ipa_subca:
    ipa_host: spider.example.com
    ipa_pass: Passw0rd!
    state: disable
    subca_name: AnsibleSubCA1
"""

RETURN = r"""
subca:
  description: IPA Sub CA record as returned by IPA API.
  returned: always
  type: dict
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec
from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


class SubCAIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def subca_find(self, subca_name):
        return self._post_json(method="ca_find", name=subca_name, item=None)

    def subca_add(self, subca_name=None, subject_dn=None, details=None):
        item = dict(ipacasubjectdn=subject_dn)
        subca_desc = details.get("description", None)
        if subca_desc is not None:
            item.update(description=subca_desc)
        return self._post_json(method="ca_add", name=subca_name, item=item)

    def subca_mod(self, subca_name=None, diff=None, details=None):
        item = get_subca_dict(details)
        for change in diff:
            update_detail = dict()
            if item[change] is not None:
                update_detail.update(setattr=f"{change}={item[change]}")
                self._post_json(method="ca_mod", name=subca_name, item=update_detail)

    def subca_del(self, subca_name=None):
        return self._post_json(method="ca_del", name=subca_name)

    def subca_disable(self, subca_name=None):
        return self._post_json(method="ca_disable", name=subca_name)

    def subca_enable(self, subca_name=None):
        return self._post_json(method="ca_enable", name=subca_name)


def get_subca_dict(details=None):
    module_subca = dict()
    if details["description"] is not None:
        module_subca["description"] = details["description"]
    if details["subca_subject"] is not None:
        module_subca["ipacasubjectdn"] = details["subca_subject"]
    return module_subca


def get_subca_diff(client, ipa_subca, module_subca):
    details = get_subca_dict(module_subca)
    return client.get_diff(ipa_data=ipa_subca, module_data=details)


def ensure(module, client):
    subca_name = module.params["subca_name"]
    subca_subject_dn = module.params["subca_subject"]
    subca_desc = module.params["subca_desc"]

    state = module.params["state"]

    ipa_subca = client.subca_find(subca_name)
    module_subca = dict(description=subca_desc, subca_subject=subca_subject_dn)

    changed = False
    if state == "present":
        if not ipa_subca:
            changed = True
            if not module.check_mode:
                client.subca_add(subca_name=subca_name, subject_dn=subca_subject_dn, details=module_subca)
        else:
            diff = get_subca_diff(client, ipa_subca, module_subca)
            # IPA does not allow to modify Sub CA's subject DN
            # So skip it for now.
            if "ipacasubjectdn" in diff:
                diff.remove("ipacasubjectdn")
                del module_subca["subca_subject"]

            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    client.subca_mod(subca_name=subca_name, diff=diff, details=module_subca)
    elif state == "absent":
        if ipa_subca:
            changed = True
            if not module.check_mode:
                client.subca_del(subca_name=subca_name)
    elif state == "disable":
        ipa_version = client.get_ipa_version()
        if LooseVersion(ipa_version) < LooseVersion("4.4.2"):
            module.fail_json(
                msg="Current version of IPA server [%s] does not support 'CA disable' option. Please upgrade to "
                "version greater than 4.4.2"
            )
        if ipa_subca:
            changed = True
            if not module.check_mode:
                client.subca_disable(subca_name=subca_name)
    elif state == "enable":
        ipa_version = client.get_ipa_version()
        if LooseVersion(ipa_version) < LooseVersion("4.4.2"):
            module.fail_json(
                msg="Current version of IPA server [%s] does not support 'CA enable' option. Please upgrade to "
                "version greater than 4.4.2"
            )
        if ipa_subca:
            changed = True
            if not module.check_mode:
                client.subca_enable(subca_name=subca_name)

    return changed, client.subca_find(subca_name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        subca_name=dict(type="str", required=True, aliases=["name"]),
        subca_subject=dict(type="str", required=True),
        subca_desc=dict(type="str"),
        state=dict(type="str", default="present", choices=["present", "absent", "enabled", "disabled"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    client = SubCAIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )

    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, record = ensure(module, client)
        module.exit_json(changed=changed, record=record)
    except Exception as exc:
        module.fail_json(msg=f"{exc}")


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: ipa_group
author: Thomas Krahn (@Nosmoht)
short_description: Manage FreeIPA group
description:
  - Add, modify and delete group within IPA server.
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  append:
    description:
      - If V(true), add the listed O(user) and O(group) to the group members.
      - If V(false), only the listed O(user) and O(group) are set as group members, removing any other members.
    default: false
    type: bool
    version_added: 4.0.0
  cn:
    description:
      - Canonical name.
      - Can not be changed as it is the unique identifier.
    required: true
    aliases: ['name']
    type: str
  description:
    description:
      - Description of the group.
    type: str
  external:
    description:
      - Allow adding external non-IPA members from trusted domains.
    type: bool
  gidnumber:
    description:
      - GID (use this option to set it manually).
    aliases: ['gid']
    type: str
  group:
    description:
      - List of group names assigned to this group.
      - If O(append=false) and an empty list is passed all groups are removed from this group.
      - Groups that are already assigned but not passed are removed.
      - If O(append=true) the listed groups are assigned without removing other groups.
      - If option is omitted assigned groups are not checked or changed.
    type: list
    elements: str
  nonposix:
    description:
      - Create as a non-POSIX group.
    type: bool
  user:
    description:
      - List of user names assigned to this group.
      - If O(append=false) and an empty list is passed all users are removed from this group.
      - Users that are already assigned but not passed are removed.
      - If O(append=true) the listed users are assigned without removing other users.
      - If option is omitted assigned users are not checked or changed.
    type: list
    elements: str
  external_user:
    description:
      - List of external users assigned to this group.
      - Behaves identically to O(user) with respect to O(append) attribute.
      - List entries can be in V(DOMAIN\\\\username) or SID format.
      - Unless SIDs are provided, the module always attempts to make changes even if the group already has all the users.
        This is because only SIDs are returned by IPA query.
      - O(external=true) is needed for this option to work.
    type: list
    elements: str
    version_added: 6.3.0
  state:
    description:
      - State to ensure.
    default: "present"
    choices: ["absent", "present"]
    type: str
extends_documentation_fragment:
  - community.general._ipa.documentation
  - community.general._ipa.connection_notes
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Ensure group is present
  community.general.ipa_group:
    name: oinstall
    gidnumber: '54321'
    state: present
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure that groups sysops and appops are assigned to ops but no other group
  community.general.ipa_group:
    name: ops
    group:
      - sysops
      - appops
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure that users linus and larry are assign to the group, but no other user
  community.general.ipa_group:
    name: sysops
    user:
      - linus
      - larry
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure that new starter named john is member of the group, without removing other members
  community.general.ipa_group:
    name: developers
    user:
      - john
    append: true
    state: present
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Add external user to a group
  community.general.ipa_group:
    name: developers
    external: true
    append: true
    external_user:
      - S-1-5-21-123-1234-12345-63421
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Add a user from MYDOMAIN
  community.general.ipa_group:
    name: developers
    external: true
    append: true
    external_user:
      - MYDOMAIN\\john
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret

- name: Ensure group is absent
  community.general.ipa_group:
    name: sysops
    state: absent
    ipa_host: ipa.example.com
    ipa_user: admin
    ipa_pass: topsecret
"""

RETURN = r"""
group:
  description: Group as returned by IPA API.
  returned: always
  type: dict
"""

import traceback

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._ipa import IPAClient, ipa_argument_spec


class GroupIPAClient(IPAClient):
    def __init__(self, module, host, port, protocol):
        super().__init__(module, host, port, protocol)

    def group_find(self, name):
        return self._post_json(method="group_find", name=None, item={"all": True, "cn": name})

    def group_add(self, name, item):
        return self._post_json(method="group_add", name=name, item=item)

    def group_mod(self, name, item):
        return self._post_json(method="group_mod", name=name, item=item)

    def group_del(self, name):
        return self._post_json(method="group_del", name=name)

    def group_add_member(self, name, item):
        return self._post_json(method="group_add_member", name=name, item=item)

    def group_add_member_group(self, name, item):
        return self.group_add_member(name=name, item={"group": item})

    def group_add_member_user(self, name, item):
        return self.group_add_member(name=name, item={"user": item})

    def group_add_member_externaluser(self, name, item):
        return self.group_add_member(name=name, item={"ipaexternalmember": item})

    def group_remove_member(self, name, item):
        return self._post_json(method="group_remove_member", name=name, item=item)

    def group_remove_member_group(self, name, item):
        return self.group_remove_member(name=name, item={"group": item})

    def group_remove_member_user(self, name, item):
        return self.group_remove_member(name=name, item={"user": item})

    def group_remove_member_externaluser(self, name, item):
        return self.group_remove_member(name=name, item={"ipaexternalmember": item})


def get_group_dict(description=None, external=None, gid=None, nonposix=None):
    group = {}
    if description is not None:
        group["description"] = description
    if external is not None:
        group["external"] = external
    if gid is not None:
        group["gidnumber"] = gid
    if nonposix is not None:
        group["nonposix"] = nonposix
    return group


def get_group_diff(client, ipa_group, module_group):
    # With group_add attribute nonposix is passed, whereas with group_mod only posix can be passed.
    if "nonposix" in module_group:
        # Only non-posix groups can be changed to posix
        if not module_group["nonposix"] and ipa_group.get("nonposix"):
            module_group["posix"] = True
        del module_group["nonposix"]

    if "external" in module_group:
        is_external_in_ipa = "ipaexternalgroup" in ipa_group.get("objectclass", [])
        if module_group["external"] == is_external_in_ipa:
            del module_group["external"]

    return client.get_diff(ipa_data=ipa_group, module_data=module_group)


def ensure(module, client):
    state = module.params["state"]
    name = module.params["cn"]
    group = module.params["group"]
    user = module.params["user"]
    external = module.params["external"]
    external_user = module.params["external_user"]
    append = module.params["append"]

    module_group = get_group_dict(
        description=module.params["description"],
        external=external,
        gid=module.params["gidnumber"],
        nonposix=module.params["nonposix"],
    )
    ipa_group = client.group_find(name=name)

    if not (external or external_user is None):
        module.fail_json("external_user can only be set if external = True")

    changed = False
    if state == "present":
        if not ipa_group:
            changed = True
            if not module.check_mode:
                ipa_group = client.group_add(name, item=module_group)
        else:
            diff = get_group_diff(client, ipa_group, module_group)
            if len(diff) > 0:
                changed = True
                if not module.check_mode:
                    data = {}
                    for key in diff:
                        data[key] = module_group.get(key)
                    client.group_mod(name=name, item=data)

        if group is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_group.get("member_group", []),
                    group,
                    client.group_add_member_group,
                    client.group_remove_member_group,
                    append=append,
                )
                or changed
            )

        if user is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_group.get("member_user", []),
                    user,
                    client.group_add_member_user,
                    client.group_remove_member_user,
                    append=append,
                )
                or changed
            )

        if external_user is not None:
            changed = (
                client.modify_if_diff(
                    name,
                    ipa_group.get("ipaexternalmember", []),
                    external_user,
                    client.group_add_member_externaluser,
                    client.group_remove_member_externaluser,
                    append=append,
                )
                or changed
            )
    else:
        if ipa_group:
            changed = True
            if not module.check_mode:
                client.group_del(name)

    return changed, client.group_find(name=name)


def main():
    argument_spec = ipa_argument_spec()
    argument_spec.update(
        cn=dict(type="str", required=True, aliases=["name"]),
        description=dict(type="str"),
        external=dict(type="bool"),
        external_user=dict(type="list", elements="str"),
        gidnumber=dict(type="str", aliases=["gid"]),
        group=dict(type="list", elements="str"),
        nonposix=dict(type="bool"),
        state=dict(type="str", default="present", choices=["present", "absent"]),
        user=dict(type="list", elements="str"),
        append=dict(type="bool", default=False),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    client = GroupIPAClient(
        module=module,
        host=module.params["ipa_host"],
        port=module.params["ipa_port"],
        protocol=module.params["ipa_prot"],
    )
    try:
        client.login(username=module.params["ipa_user"], password=module.params["ipa_pass"])
        changed, group = ensure(module, client)
        module.exit_json(changed=changed, group=group)
    except Exception as e:
        module.fail_json(msg=f"{e}", exception=traceback.format_exc())


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2023, Anvitha Jain (@anvitha-jain) <anvjain@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}

DOCUMENTATION = r"""
---
module: mso_schema_site_anp_epg_bulk_staticport
short_description: Manage site-local EPG static ports in bulk in schema template
description:
- Manage site-local EPG static ports in bulk in schema template on Cisco ACI Multi-Site.
author:
- Anvitha Jain (@anvjain)
options:
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  site:
    description:
    - The name of the site.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  anp:
    description:
    - The name of the ANP.
    type: str
    required: true
  epg:
    description:
    - The name of the EPG.
    type: str
    required: true
  type:
    description:
    - The path type of the static port
    - vpc is used for a Virtual Port Channel
    - dpc is used for a Direct Port Channel
    - port is used for a single interface
    type: str
    choices: [ port, vpc, dpc ]
    default: port
  pod:
    description:
    - The pod of the static port.
    type: str
  leaf:
    description:
    - The leaf of the static port.
    type: str
  fex:
    description:
    - The fex id of the static port.
    type: str
  path:
    description:
    - The path of the static port.
    type: str
  vlan:
    description:
    - The port encap VLAN id of the static port.
    type: int
  deployment_immediacy:
    description:
    - The deployment immediacy of the static port.
    - C(immediate) means B(Deploy immediate).
    - C(lazy) means B(deploy on demand).
    type: str
    choices: [ immediate, lazy ]
    default: lazy
  mode:
    description:
    - The mode of the static port.
    - C(native) means B(Access (802.1p)).
    - C(regular) means B(Trunk).
    - C(untagged) means B(Access (untagged)).
    type: str
    choices: [ native, regular, untagged ]
    default: untagged
  primary_micro_segment_vlan:
    description:
    - Primary micro-seg VLAN of static port.
    type: int
  static_ports:
    description:
    - List of static port configurations and elements in the form of a dictionary.
    - Module level attributes will be overridden by the path level attributes.
    - Making changes to an item in the list will update the whole payload.
    type: list
    elements: dict
    suboptions:
      type:
        description:
        - The path type of the static port
        - vpc is used for a Virtual Port Channel
        - dpc is used for a Direct Port Channel
        - port is used for a single interface
        type: str
        choices: [ port, vpc, dpc ]
      pod:
        description:
        - The pod of the static port.
        type: str
      leaf:
        description:
        - The leaf of the static port.
        type: str
      fex:
        description:
        - The fex id of the static port.
        type: str
      path:
        description:
        - The path of the static port.
        - Path has to be unique for each static port in a particular leaf.
        type: str
      vlan:
        description:
        - The port encap VLAN id of the static port.
        type: int
      deployment_immediacy:
        description:
        - The deployment immediacy of the static port.
        - C(immediate) means B(Deploy immediate).
        - C(lazy) means B(deploy on demand).
        type: str
        choices: [ immediate, lazy ]
      mode:
        description:
        - The mode of the static port.
        - C(native) means B(Access (802.1p)).
        - C(regular) means B(Trunk).
        - C(untagged) means B(Access (untagged)).
        type: str
        choices: [ native, regular, untagged ]
      primary_micro_segment_vlan:
        description:
        - Primary micro-seg VLAN of the static port.
        type: int
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
notes:
- The ACI MultiSite PATCH API has a deficiency requiring some objects to be referenced by index.
  This can cause silent corruption on concurrent access when changing/removing an object as
  the wrong object may be referenced. This module is affected by this deficiency.
seealso:
- module: cisco.mso.mso_schema_site_anp_epg
- module: cisco.mso.mso_schema_template_anp_epg
deprecated:
  removed_in: 3.0.0
  alternative: Use M(cisco.mso.mso_schema_site_anp_epg_staticport) with option `force_replace=true` instead.
  why: The module has been merged to centralise all static port functionality into M(cisco.mso.mso_schema_site_anp_epg_staticport).
extends_documentation_fragment: cisco.mso.modules
"""

EXAMPLES = r"""
- name: Add a new static port to a site EPG
  cisco.mso.mso_schema_site_anp_epg_bulk_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    path: eth1/1
    vlan: 126
    deployment_immediacy: immediate
    static_ports:
      - path: eth1/2
        leaf: 102
      - path: eth1/3
        vlan: 124
    state: present

- name: Add a new static fex port to a site EPG
  cisco.mso.mso_schema_site_anp_epg_bulk_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    path: eth1/1
    vlan: 126
    deployment_immediacy: lazy
    static_ports:
      - path: eth1/2
        leaf: 102
      - path: eth1/3
        vlan: 124
      - fex: 151
    state: present

- name: Add a new static VPC to a site EPG
  cisco.mso.mso_schema_site_anp_epg_bulk_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    type: port
    pod: pod-1
    leaf: 101
    path: eth1/1
    vlan: 126
    static_ports:
      - path: eth1/2
        leaf: 102
      - path: eth1/3
        vlan: 124
      - fex: 151
      - leaf: 101-102
        path: ansible_polgrp
        vlan: 127
        type: vpc
        mode: untagged
        deployment_immediacy: lazy
    state: present

- name: Remove static ports from a site EPG
  cisco.mso.mso_schema_site_anp_epg_bulk_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    epg: EPG1
    state: absent

- name: Query all site EPG static ports
  cisco.mso.mso_schema_site_anp_epg_bulk_staticport:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: Schema1
    site: Site1
    template: Template1
    anp: ANP1
    state: query
  register: query_result
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_site_anp_epg_bulk_staticport_spec
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        schema=dict(type="str", required=True),
        site=dict(type="str", required=True),
        template=dict(type="str", required=True),
        anp=dict(type="str", required=True),
        epg=dict(type="str", required=True),
        type=dict(type="str", default="port", choices=["port", "vpc", "dpc"]),
        pod=dict(type="str"),  # This parameter is not required for querying all objects
        leaf=dict(type="str"),  # This parameter is not required for querying all objects
        fex=dict(type="str"),  # This parameter is not required for querying all objects
        path=dict(type="str"),  # This parameter is not required for querying all objects
        vlan=dict(type="int"),  # This parameter is not required for querying all objects
        primary_micro_segment_vlan=dict(type="int"),  # This parameter is not required for querying all objects
        deployment_immediacy=dict(type="str", default="lazy", choices=["immediate", "lazy"]),
        mode=dict(type="str", default="untagged", choices=["native", "regular", "untagged"]),
        static_ports=dict(type="list", elements="dict", options=mso_site_anp_epg_bulk_staticport_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "present", ["static_ports"]],
        ],
    )

    schema = module.params.get("schema")
    site = module.params.get("site")
    template = module.params.get("template").replace(" ", "")
    anp = module.params.get("anp")
    epg = module.params.get("epg")
    module_path_type = module.params.get("type")
    module_pod = module.params.get("pod")
    module_leaf = module.params.get("leaf")
    module_fex = module.params.get("fex")
    module_path = module.params.get("path")
    module_vlan = module.params.get("vlan")
    module_primary_micro_segment_vlan = module.params.get("primary_micro_segment_vlan")
    module_deployment_immediacy = module.params.get("deployment_immediacy")
    module_mode = module.params.get("mode")
    static_ports = module.params.get("static_ports")
    state = module.params.get("state")

    mso = MSOModule(module)

    # Get schema objects
    mso_schema = MSOSchema(mso, schema, template, site)
    mso_objects = mso_schema.schema_objects

    # Verifies ANP  and EPG exists at template level
    mso_schema.set_template_anp(anp)
    mso_schema.set_template_anp_epg(epg)

    # Verifies if ANP exists at site level
    mso_schema.set_site_anp(anp, fail_module=False)

    payload = dict()
    ops = []
    op_path = "/sites/{0}-{1}/anps".format(mso_objects.get("site").details.get("siteId"), template)
    mso.existing = []

    # If anp not at site level but exists at template level
    if not mso_objects.get("site_anp"):
        op_path = op_path + "/-"
        payload.update(
            anpRef=dict(
                schemaId=mso_schema.id,
                templateName=template,
                anpName=anp,
            ),
        )
    else:
        mso_schema.set_site_anp_epg(epg, fail_module=False)

    # If epg not at site level (or) anp not at site level payload
    if not mso_objects.get("site_anp_epg") or "anpRef" in payload:
        # EPG at template level but not at site level. Create payload at site level for EPG
        new_epg = dict(
            epgRef=dict(
                schemaId=mso_schema.id,
                templateName=template,
                anpName=anp,
                epgName=epg,
            )
        )

        # If anp not in payload then, anp already exists at site level. New payload will only have new EPG payload
        if "anpRef" not in payload:
            op_path = "{0}/{1}/epgs/-".format(op_path, anp)
            payload = new_epg
        else:
            # If anp in payload, anp exists at site level. Update payload with EPG payload
            payload["epgs"] = [new_epg]
    else:
        # If anp and epg exists at site level
        op_path = "{0}/{1}/epgs/{2}/staticPorts".format(op_path, anp, epg)
        mso.existing = mso_objects.get("site_anp_epg").details.get("staticPorts", [])

    staticport_list = []
    unique_paths = []

    mso.previous = mso.existing

    if state == "absent":
        if mso.existing:
            mso.sent = mso.existing = []
            ops.append(dict(op="remove", path=op_path))

    elif state == "present":
        for static_port in static_ports:
            path_type = static_port.get("type") or module_path_type
            pod = static_port.get("pod") or module_pod
            leaf = static_port.get("leaf") or module_leaf
            fex = static_port.get("fex") or module_fex
            path = static_port.get("path") or module_path  # Note :path has to be diffent in each leaf for every static port in the list.
            vlan = static_port.get("vlan") or module_vlan
            primary_micro_segment_vlan = static_port.get("primary_micro_segment_vlan") or module_primary_micro_segment_vlan
            deployment_immediacy = static_port.get("deployment_immediacy") or module_deployment_immediacy
            mode = static_port.get("mode") or module_mode

            required_dict = {"pod": pod, "leaf": leaf, "path": path, "vlan": vlan}
            if None in required_dict.values():
                res = [key for key in required_dict.keys() if required_dict[key] is None]
                mso.fail_json(msg="state is present but all of the following are missing: {0}.".format(", ".join(res)))
            else:
                if path_type == "port" and fex is not None:
                    # Select port path for fex if fex param is used
                    portpath = "topology/{0}/paths-{1}/extpaths-{2}/pathep-[{3}]".format(pod, leaf, fex, path)
                elif path_type == "vpc":
                    portpath = "topology/{0}/protpaths-{1}/pathep-[{2}]".format(pod, leaf, path)
                else:
                    portpath = "topology/{0}/paths-{1}/pathep-[{2}]".format(pod, leaf, path)

            new_leaf = dict(
                deploymentImmediacy=deployment_immediacy,
                mode=mode,
                path=portpath,
                portEncapVlan=vlan,
                type=path_type,
            )

            if primary_micro_segment_vlan:
                new_leaf.update(microSegVlan=primary_micro_segment_vlan)

            # validate and append staticports to staticport_list if path variable is different
            if portpath in unique_paths:
                mso.fail_json(msg="Each leaf in a pod of a static port should have an unique path.")
            else:
                unique_paths.append(portpath)
                staticport_list.append(new_leaf)

        # If payload is empty, anp and EPG already exist at site level
        if not payload:
            payload = staticport_list
        elif "anpRef" not in payload:  # If anp already exists at site level
            payload["staticPorts"] = staticport_list
        else:
            payload["epgs"][0]["staticPorts"] = staticport_list

        mso.proposed = staticport_list
        mso.sent = payload

        if mso.existing:
            ops.append(dict(op="replace", path=op_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=op_path, value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode and mso.proposed != mso.previous:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python

# Copyright (c) 2016, Ben Doherty <bendohmv@gmail.com>
# Sponsored by Oomph, Inc. http://www.oomphinc.com
# Copyright (c) 2017, Ansible Project
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: archive
short_description: Creates a compressed archive of one or more files or trees
extends_documentation_fragment:
  - ansible.builtin.files
  - community.general._attributes
description:
  - Creates or extends an archive.
  - The source and archive are on the target host, and the archive I(is not) copied to the controller host.
  - Source files can be deleted after archival by specifying O(remove=True).
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  path:
    description:
      - Remote absolute path, glob, or list of paths or globs for the file or files to compress or archive.
    type: list
    elements: path
    required: true
  format:
    description:
      - The type of compression to use.
    type: str
    choices: [bz2, gz, tar, xz, zip]
    default: gz
  dest:
    description:
      - The file name of the destination archive. The parent directory must exists on the remote host.
      - This is required when O(path) refers to multiple files by either specifying a glob, a directory or multiple paths
        in a list.
      - If the destination archive already exists, it is truncated and overwritten.
    type: path
  exclude_path:
    description:
      - Remote absolute path, glob, or list of paths or globs for the file or files to exclude from O(path) list and glob
        expansion.
      - Use O(exclusion_patterns) to instead exclude files or subdirectories below any of the paths from the O(path) list.
    type: list
    elements: path
    default: []
  exclusion_patterns:
    description:
      - Glob style patterns to exclude files or directories from the resulting archive.
      - This differs from O(exclude_path) which applies only to the source paths from O(path).
    type: list
    elements: path
    version_added: 3.2.0
  force_archive:
    description:
      - Allows you to force the module to treat this as an archive even if only a single file is specified.
      - By default when a single file is specified it is compressed only (not archived).
      - Enable this if you want to use M(ansible.builtin.unarchive) on an archive of a single file created with this module.
    type: bool
    default: false
  remove:
    description:
      - Remove any added source files and trees after adding to archive.
    type: bool
    default: false
notes:
  - Can produce C(gzip), C(bzip2), C(lzma), and C(zip) compressed files or archives.
  - This module uses C(tarfile), C(zipfile), C(gzip), C(bz2), and C(lzma) packages on the target host to create archives. These are
    part of the Python standard library.
seealso:
  - module: ansible.builtin.unarchive
author:
  - Ben Doherty (@bendoh)
"""

EXAMPLES = r"""
- name: Compress directory /path/to/foo/ into /path/to/foo.tgz
  community.general.archive:
    path: /path/to/foo
    dest: /path/to/foo.tgz

- name: Compress regular file /path/to/foo into /path/to/foo.gz and remove it
  community.general.archive:
    path: /path/to/foo
    remove: true

- name: Create a zip archive of /path/to/foo
  community.general.archive:
    path: /path/to/foo
    format: zip

- name: Create a bz2 archive of multiple files, rooted at /path
  community.general.archive:
    path:
      - /path/to/foo
      - /path/wong/foo
    dest: /path/file.tar.bz2
    format: bz2

- name: Create a bz2 archive of a globbed path, while excluding specific dirnames
  community.general.archive:
    path:
      - /path/to/foo/*
    dest: /path/file.tar.bz2
    exclude_path:
      - /path/to/foo/bar
      - /path/to/foo/baz
    format: bz2

- name: Create a bz2 archive of a globbed path, while excluding a glob of dirnames
  community.general.archive:
    path:
      - /path/to/foo/*
    dest: /path/file.tar.bz2
    exclude_path:
      - /path/to/foo/ba*
    format: bz2

- name: Use gzip to compress a single archive (i.e don't archive it first with tar)
  community.general.archive:
    path: /path/to/foo/single.file
    dest: /path/file.gz
    format: gz

- name: Create a tar.gz archive of a single file.
  community.general.archive:
    path: /path/to/foo/single.file
    dest: /path/file.tar.gz
    format: gz
    force_archive: true
"""

RETURN = r"""
state:
  description: The state of the input O(path).
  type: str
  returned: always
dest_state:
  description:
    - The state of the O(dest) file.
    - V(absent) when the file does not exist.
    - V(archive) when the file is an archive.
    - V(compress) when the file is compressed, but not an archive.
    - V(incomplete) when the file is an archive, but some files under O(path) were not found.
  type: str
  returned: success
  version_added: 3.4.0
missing:
  description: Any files that were missing from the source.
  type: list
  returned: success
archived:
  description: Any files that were compressed or added to the archive.
  type: list
  returned: success
arcroot:
  description: The archive root.
  type: str
  returned: always
expanded_paths:
  description: The list of matching paths from paths argument.
  type: list
  returned: always
expanded_exclude_paths:
  description: The list of matching exclude paths from the exclude_path argument.
  type: list
  returned: always
"""

import abc
import bz2
import glob
import gzip
import io
import lzma
import os
import re
import shutil
import tarfile
import zipfile
from fnmatch import fnmatch
from traceback import format_exc
from zipfile import BadZipFile
from zlib import crc32

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.common.text.converters import to_bytes, to_native

STATE_ABSENT = "absent"
STATE_ARCHIVED = "archive"
STATE_COMPRESSED = "compress"
STATE_INCOMPLETE = "incomplete"


def common_path(paths):
    empty = b"" if paths and isinstance(paths[0], bytes) else ""

    return os.path.join(
        os.path.dirname(os.path.commonprefix([os.path.join(os.path.dirname(p), empty) for p in paths])), empty
    )


def expand_paths(paths):
    expanded_path = []
    is_globby = False
    for path in paths:
        b_path = _to_bytes(path)
        if b"*" in b_path or b"?" in b_path:
            e_paths = glob.glob(b_path)
            is_globby = True
        else:
            e_paths = [b_path]
        expanded_path.extend(e_paths)
    return expanded_path, is_globby


def matches_exclusion_patterns(path, exclusion_patterns):
    return any(fnmatch(path, p) for p in exclusion_patterns)


def is_archive(path):
    return re.search(rb"\.(tar|tar\.(gz|bz2|xz)|tgz|tbz2|zip)$", os.path.basename(path), re.IGNORECASE)


def strip_prefix(prefix, string):
    return string[len(prefix) :] if string.startswith(prefix) else string


def _to_bytes(s):
    return to_bytes(s, errors="surrogate_or_strict")


def _to_native(s):
    return to_native(s, errors="surrogate_or_strict")


def _to_native_ascii(s):
    return to_native(s, errors="surrogate_or_strict", encoding="ascii")


class Archive(metaclass=abc.ABCMeta):
    def __init__(self, module):
        self.module = module

        self.destination = _to_bytes(module.params["dest"]) if module.params["dest"] else None
        self.exclusion_patterns = module.params["exclusion_patterns"] or []
        self.format = module.params["format"]
        self.must_archive = module.params["force_archive"]
        self.remove = module.params["remove"]

        self.changed = False
        self.destination_state = STATE_ABSENT
        self.errors = []
        self.file = None
        self.successes = []
        self.targets = []
        self.not_found = []

        paths = module.params["path"]
        self.expanded_paths, has_globs = expand_paths(paths)
        self.expanded_exclude_paths = expand_paths(module.params["exclude_path"])[0]

        self.paths = sorted(set(self.expanded_paths) - set(self.expanded_exclude_paths))

        if not self.paths:
            module.fail_json(
                path=", ".join(paths),
                expanded_paths=_to_native(b", ".join(self.expanded_paths)),
                expanded_exclude_paths=_to_native(b", ".join(self.expanded_exclude_paths)),
                msg="Error, no source paths were found",
            )

        self.root = common_path(self.paths)

        if not self.must_archive:
            self.must_archive = any([has_globs, os.path.isdir(self.paths[0]), len(self.paths) > 1])

        if not self.destination and not self.must_archive:
            self.destination = b"%s.%s" % (self.paths[0], _to_bytes(self.format))

        if self.must_archive and not self.destination:
            module.fail_json(
                dest=_to_native(self.destination),
                path=", ".join(paths),
                msg='Error, must specify "dest" when archiving multiple files or trees',
            )

        if self.remove:
            self._check_removal_safety()

        self.original_checksums = self.destination_checksums()
        self.original_size = self.destination_size()

    def add(self, path, archive_name):
        try:
            self._add(_to_native_ascii(path), _to_native(archive_name))
            if self.contains(_to_native(archive_name)):
                self.successes.append(path)
        except Exception as e:
            self.errors.append(f"{_to_native_ascii(path)}: {e}")

    def add_single_target(self, path):
        if self.format in ("zip", "tar"):
            self.open()
            self.add(path, strip_prefix(self.root, path))
            self.close()
            self.destination_state = STATE_ARCHIVED
        else:
            try:
                f_out = self._open_compressed_file(_to_native_ascii(self.destination), "wb")
                with open(path, "rb") as f_in:
                    shutil.copyfileobj(f_in, f_out)
                f_out.close()
                self.successes.append(path)
                self.destination_state = STATE_COMPRESSED
            except OSError as e:
                self.module.fail_json(
                    path=_to_native(path),
                    dest=_to_native(self.destination),
                    msg=f"Unable to write to compressed file: {e}",
                    exception=format_exc(),
                )

    def add_targets(self):
        self.open()
        try:
            for target in self.targets:
                if os.path.isdir(target):
                    for directory_path, directory_names, file_names in os.walk(target, topdown=True):
                        for directory_name in directory_names:
                            full_path = os.path.join(directory_path, directory_name)
                            self.add(full_path, strip_prefix(self.root, full_path))

                        for file_name in file_names:
                            full_path = os.path.join(directory_path, file_name)
                            self.add(full_path, strip_prefix(self.root, full_path))
                else:
                    self.add(target, strip_prefix(self.root, target))
        except Exception as e:
            if self.format in ("zip", "tar"):
                archive_format = self.format
            else:
                archive_format = f"tar.{self.format}"
            self.module.fail_json(
                msg=f"Error when writing {archive_format} archive at {_to_native(self.destination)}: {e}",
                exception=format_exc(),
            )
        self.close()

        if self.errors:
            self.module.fail_json(
                msg=f"Errors when writing archive at {_to_native(self.destination)}: {'; '.join(self.errors)}"
            )

    def is_different_from_original(self):
        if self.original_checksums is None:
            return self.original_size != self.destination_size()
        else:
            return self.original_checksums != self.destination_checksums()

    def destination_checksums(self):
        if self.destination_exists() and self.destination_readable():
            return self._get_checksums(self.destination)
        return None

    def destination_exists(self):
        return self.destination and os.path.exists(self.destination)

    def destination_readable(self):
        return self.destination and os.access(self.destination, os.R_OK)

    def destination_size(self):
        return os.path.getsize(self.destination) if self.destination_exists() else 0

    def find_targets(self):
        for path in self.paths:
            if not os.path.lexists(path):
                self.not_found.append(path)
            else:
                self.targets.append(path)

    def has_targets(self):
        return bool(self.targets)

    def has_unfound_targets(self):
        return bool(self.not_found)

    def remove_single_target(self, path):
        try:
            os.remove(path)
        except OSError as e:
            self.module.fail_json(
                path=_to_native(path), msg=f"Unable to remove source file: {e}", exception=format_exc()
            )

    def remove_targets(self):
        for path in self.successes:
            if os.path.exists(path):
                try:
                    if os.path.isdir(path):
                        shutil.rmtree(path)
                    else:
                        os.remove(path)
                except OSError:
                    self.errors.append(_to_native(path))
        for path in self.paths:
            try:
                if os.path.isdir(path):
                    shutil.rmtree(path)
            except OSError:
                self.errors.append(_to_native(path))

        if self.errors:
            self.module.fail_json(
                dest=_to_native(self.destination), msg="Error deleting some source files: ", files=self.errors
            )

    def update_permissions(self):
        file_args = self.module.load_file_common_arguments(self.module.params, path=self.destination)
        self.changed = self.module.set_fs_attributes_if_different(file_args, self.changed)

    @property
    def result(self):
        return {
            "archived": [_to_native(p) for p in self.successes],
            "dest": _to_native(self.destination),
            "dest_state": self.destination_state,
            "changed": self.changed,
            "arcroot": _to_native(self.root),
            "missing": [_to_native(p) for p in self.not_found],
            "expanded_paths": [_to_native(p) for p in self.expanded_paths],
            "expanded_exclude_paths": [_to_native(p) for p in self.expanded_exclude_paths],
        }

    def _check_removal_safety(self):
        for path in self.paths:
            if os.path.isdir(path) and self.destination.startswith(os.path.join(path, b"")):
                self.module.fail_json(
                    path=b", ".join(self.paths),
                    msg="Error, created archive can not be contained in source paths when remove=true",
                )

    def _open_compressed_file(self, path, mode):
        f = None
        if self.format == "gz":
            f = gzip.open(path, mode)
        elif self.format == "bz2":
            f = bz2.BZ2File(path, mode)
        elif self.format == "xz":
            f = lzma.LZMAFile(path, mode)
        else:
            self.module.fail_json(msg=f"{self.format} is not a valid format")

        return f

    @abc.abstractmethod
    def close(self):
        pass

    @abc.abstractmethod
    def contains(self, name):
        pass

    @abc.abstractmethod
    def open(self):
        pass

    @abc.abstractmethod
    def _add(self, path, archive_name):
        pass

    @abc.abstractmethod
    def _get_checksums(self, path):
        pass


class ZipArchive(Archive):
    def __init__(self, module):
        super().__init__(module)

    def close(self):
        self.file.close()

    def contains(self, name):
        try:
            self.file.getinfo(name)
        except KeyError:
            return False
        return True

    def open(self):
        self.file = zipfile.ZipFile(_to_native_ascii(self.destination), "w", zipfile.ZIP_DEFLATED, True)

    def _add(self, path, archive_name):
        if not matches_exclusion_patterns(path, self.exclusion_patterns):
            self.file.write(path, archive_name)

    def _get_checksums(self, path):
        try:
            archive = zipfile.ZipFile(_to_native_ascii(path), "r")
            checksums = {(info.filename, info.CRC) for info in archive.infolist()}
            archive.close()
        except BadZipFile:
            checksums = set()
        return checksums


class TarArchive(Archive):
    def __init__(self, module):
        super().__init__(module)
        self.fileIO = None

    def close(self):
        self.file.close()
        if self.format == "xz":
            with lzma.open(_to_native(self.destination), "wb") as f:
                f.write(self.fileIO.getvalue())
            self.fileIO.close()

    def contains(self, name):
        try:
            self.file.getmember(name)
        except KeyError:
            return False
        return True

    def open(self):
        if self.format in ("gz", "bz2"):
            self.file = tarfile.open(_to_native_ascii(self.destination), f"w|{self.format}")
        # python3 tarfile module allows xz format but for python2 we have to create the tarfile
        # in memory and then compress it with lzma.
        elif self.format == "xz":
            self.fileIO = io.BytesIO()
            self.file = tarfile.open(fileobj=self.fileIO, mode="w")
        elif self.format == "tar":
            self.file = tarfile.open(_to_native_ascii(self.destination), "w")
        else:
            self.module.fail_json(msg=f"{self.format} is not a valid archive format")

    def _add(self, path, archive_name):
        def filter(tarinfo):
            return None if matches_exclusion_patterns(tarinfo.name, self.exclusion_patterns) else tarinfo

        self.file.add(path, archive_name, recursive=False, filter=filter)

    def _get_checksums(self, path):
        LZMAError = lzma.LZMAError

        try:
            if self.format == "xz":
                with lzma.open(_to_native_ascii(path), "r") as f:
                    archive = tarfile.open(fileobj=f)
                    checksums = {(info.name, info.chksum) for info in archive.getmembers()}
                    archive.close()
            else:
                archive = tarfile.open(_to_native_ascii(path), f"r|{self.format}")
                checksums = {(info.name, info.chksum) for info in archive.getmembers()}
                archive.close()
        except (LZMAError, tarfile.ReadError, tarfile.CompressionError):
            try:
                # The python implementations of gzip, bz2, and lzma do not support restoring compressed files
                # to their original names so only file checksum is returned
                f = self._open_compressed_file(_to_native_ascii(path), "r")
                checksum = 0
                while True:
                    chunk = f.read(16 * 1024 * 1024)
                    if not chunk:
                        break
                    checksum = crc32(chunk, checksum)
                checksums = {(b"", checksum)}
                f.close()
            except Exception:
                checksums = set()
        return checksums


def get_archive(module):
    if module.params["format"] == "zip":
        return ZipArchive(module)
    else:
        return TarArchive(module)


def create_module() -> AnsibleModule:
    module = AnsibleModule(
        argument_spec=dict(
            path=dict(type="list", elements="path", required=True),
            format=dict(type="str", default="gz", choices=["bz2", "gz", "tar", "xz", "zip"]),
            dest=dict(type="path"),
            exclude_path=dict(type="list", elements="path", default=[]),
            exclusion_patterns=dict(type="list", elements="path"),
            force_archive=dict(type="bool", default=False),
            remove=dict(type="bool", default=False),
        ),
        add_file_common_args=True,
        supports_check_mode=True,
    )
    return module


def main():
    module = create_module()

    check_mode = module.check_mode

    archive = get_archive(module)
    archive.find_targets()

    if not archive.has_targets():
        if archive.destination_exists():
            archive.destination_state = STATE_ARCHIVED if is_archive(archive.destination) else STATE_COMPRESSED
    elif archive.has_targets() and archive.must_archive:
        if check_mode:
            archive.changed = True
        else:
            archive.add_targets()
            archive.destination_state = STATE_INCOMPLETE if archive.has_unfound_targets() else STATE_ARCHIVED
            archive.changed |= archive.is_different_from_original()
            if archive.remove:
                archive.remove_targets()
    else:
        if check_mode:
            if not archive.destination_exists():
                archive.changed = True
        else:
            path = archive.paths[0]
            archive.add_single_target(path)
            archive.changed |= archive.is_different_from_original()
            if archive.remove:
                archive.remove_single_target(path)

    if archive.destination_exists():
        archive.update_permissions()

    module.exit_json(**archive.result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

import string
import textwrap

from ansible.module_utils.common.text.converters import to_native
from ansible.module_utils.six.moves.urllib import parse as urlparse


def _windows_callback_script(passwd=None):
    script_url = (
        "https://raw.githubusercontent.com/ansible/ansible/devel/examples/scripts/ConfigureRemotingForAnsible.ps1"
    )
    if passwd is not None:
        passwd = passwd.replace("'", "''")
        script_tpl = """\
        <powershell>
        $admin = [adsi]('WinNT://./administrator, user')
        $admin.PSBase.Invoke('SetPassword', '${PASS}')
        Invoke-Expression ((New-Object System.Net.Webclient).DownloadString('${SCRIPT}'))
        </powershell>
        """
    else:
        script_tpl = """\
        <powershell>
        $admin = [adsi]('WinNT://./administrator, user')
        Invoke-Expression ((New-Object System.Net.Webclient).DownloadString('${SCRIPT}'))
        </powershell>
        """

    tpl = string.Template(textwrap.dedent(script_tpl))
    return tpl.safe_substitute(PASS=passwd, SCRIPT=script_url)


def _linux_callback_script(tower_address, template_id, host_config_key):
    template_id = urlparse.quote(template_id)
    tower_address = urlparse.quote(tower_address)
    host_config_key = host_config_key.replace("'", "'\"'\"'")

    script_tpl = """\
    #!/bin/bash
    set -x

    retry_attempts=10
    attempt=0
    while [[ $attempt -lt $retry_attempts ]]
    do
      status_code=$(curl --max-time 10 -v -k -s -i \
        --data 'host_config_key=${host_config_key}' \
        'https://${tower_address}/api/v2/job_templates/${template_id}/callback/' \
        | head -n 1 \
        | awk '{print $2}')
      if [[ $status_code == 404 ]]
        then
        status_code=$(curl --max-time 10 -v -k -s -i \
          --data 'host_config_key=${host_config_key}' \
          'https://${tower_address}/api/v1/job_templates/${template_id}/callback/' \
          | head -n 1 \
          | awk '{print $2}')
        # fall back to using V1 API for Tower 3.1 and below, since v2 API will always 404
      fi
      if [[ $status_code == 201 ]]
        then
        exit 0
      fi
      attempt=$(( attempt + 1 ))
      echo "$${status_code} received... retrying in 1 minute. (Attempt $${attempt})"
      sleep 60
    done
    exit 1
    """
    tpl = string.Template(textwrap.dedent(script_tpl))
    return tpl.safe_substitute(tower_address=tower_address, template_id=template_id, host_config_key=host_config_key)


def tower_callback_script(tower_address, job_template_id, host_config_key, windows, passwd):
    if windows:
        return to_native(_windows_callback_script(passwd=passwd))
    return _linux_callback_script(tower_address, job_template_id, host_config_key)



#!/usr/bin/python
# Copyright (c) 2017, Daniel Korn <korndaniel1@gmail.com>
# Copyright (c) 2017, Yaacov Zamir <yzamir@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: manageiq_provider
short_description: Management of provider in ManageIQ
extends_documentation_fragment:
  - community.general._manageiq
  - community.general._attributes

author: Daniel Korn (@dkorn)
description:
  - The manageiq_provider module supports adding, updating, and deleting provider in ManageIQ.
attributes:
  check_mode:
    support: none
  diff_mode:
    support: none

options:
  state:
    type: str
    description:
      - V(absent) - provider should not exist,
      - V(present) - provider should be present,
      - V(refresh) - provider is refreshed.
    choices: ['absent', 'present', 'refresh']
    default: 'present'
  name:
    type: str
    description: The provider's name.
    required: true
  type:
    type: str
    description: The provider's type.
    choices: ['Openshift', 'Amazon', 'oVirt', 'VMware', 'Azure', 'Director', 'OpenStack', 'GCE']
  zone:
    type: str
    description: The ManageIQ zone name that manages the provider.
    default: 'default'
  provider_region:
    type: str
    description: The provider region name to connect to (for example AWS region for Amazon).
  host_default_vnc_port_start:
    type: str
    description: The first port in the host VNC range.
  host_default_vnc_port_end:
    type: str
    description: The last port in the host VNC range.
  subscription:
    type: str
    description: Microsoft Azure subscription ID.
  project:
    type: str
    description: Google Compute Engine Project ID.
  azure_tenant_id:
    type: str
    description: Tenant ID. Defaults to V(null).
    aliases: [keystone_v3_domain_id]
  tenant_mapping_enabled:
    type: bool
    default: false
    description: Whether to enable mapping of existing tenants.
  api_version:
    type: str
    description: The OpenStack Keystone API version.
    choices: ['v2', 'v3']

  provider:
    description: Default endpoint connection information, required if state is true.
    type: dict
    suboptions:
      hostname:
        type: str
        description: The provider's API hostname.
        required: true
      port:
        type: int
        description: The provider's API port.
      userid:
        type: str
        description: Provider's API endpoint authentication userid.
      password:
        type: str
        description: Provider's API endpoint authentication password.
      auth_key:
        type: str
        description: Provider's API endpoint authentication bearer token.
      validate_certs:
        description: Whether SSL certificates should be verified for HTTPS requests (deprecated).
        type: bool
        default: true
        aliases: [verify_ssl]
      security_protocol:
        type: str
        description: How SSL certificates should be used for HTTPS requests.
        choices: ['ssl-with-validation', 'ssl-with-validation-custom-ca', 'ssl-without-validation', 'non-ssl']
      certificate_authority:
        type: str
        description: The CA bundle string with custom certificates.
      path:
        type: str
        description:
          - TODO needs documentation.
      project:
        type: str
        description:
          - TODO needs documentation.
      role:
        type: str
        description:
          - TODO needs documentation.
      subscription:
        type: str
        description:
          - TODO needs documentation.
      uid_ems:
        type: str
        description:
          - TODO needs documentation.
  metrics:
    description: Metrics endpoint connection information.
    type: dict
    suboptions:
      hostname:
        type: str
        description: The provider's API hostname.
        required: true
      port:
        type: int
        description: The provider's API port.
      userid:
        type: str
        description: Provider's API endpoint authentication userid.
      password:
        type: str
        description: Provider's API endpoint authentication password.
      auth_key:
        type: str
        description: Provider's API endpoint authentication bearer token.
      validate_certs:
        description: Whether SSL certificates should be verified for HTTPS requests (deprecated).
        type: bool
        default: true
        aliases: [verify_ssl]
      security_protocol:
        type: str
        choices: ['ssl-with-validation', 'ssl-with-validation-custom-ca', 'ssl-without-validation', 'non-ssl']
        description: How SSL certificates should be used for HTTPS requests.
      certificate_authority:
        type: str
        description: The CA bundle string with custom certificates.
      path:
        type: str
        description: Database name for oVirt metrics. Defaults to V(ovirt_engine_history).
      project:
        type: str
        description:
          - TODO needs documentation.
      role:
        type: str
        description:
          - TODO needs documentation.
      subscription:
        type: str
        description:
          - TODO needs documentation.
      uid_ems:
        type: str
        description:
          - TODO needs documentation.
  alerts:
    description: Alerts endpoint connection information.
    type: dict
    suboptions:
      hostname:
        type: str
        description: The provider's API hostname.
        required: true
      port:
        type: int
        description: The provider's API port.
      userid:
        type: str
        description: Provider's API endpoint authentication userid. Defaults to V(null).
      password:
        type: str
        description: Provider's API endpoint authentication password. Defaults to V(null).
      auth_key:
        type: str
        description: Provider's API endpoint authentication bearer token. Defaults to V(null).
      validate_certs:
        type: bool
        description: Whether SSL certificates should be verified for HTTPS requests (deprecated). Defaults to V(true).
        default: true
        aliases: [verify_ssl]
      security_protocol:
        type: str
        choices: ['ssl-with-validation', 'ssl-with-validation-custom-ca', 'ssl-without-validation', 'non-ssl']
        description: How SSL certificates should be used for HTTPS requests. Defaults to V(null).
      certificate_authority:
        type: str
        description: The CA bundle string with custom certificates. Defaults to V(null).
      path:
        type: str
        description:
          - TODO needs documentation.
      project:
        type: str
        description:
          - TODO needs documentation.
      role:
        type: str
        description:
          - TODO needs documentation.
      subscription:
        type: str
        description:
          - TODO needs documentation.
      uid_ems:
        type: str
        description:
          - TODO needs documentation.
  ssh_keypair:
    description: SSH key pair used for SSH connections to all hosts in this provider.
    type: dict
    suboptions:
      hostname:
        type: str
        description: Director hostname.
        required: true
      userid:
        type: str
        description: SSH username.
      auth_key:
        type: str
        description: SSH private key.
      validate_certs:
        description:
          - Whether certificates should be verified for connections.
        type: bool
        default: true
        aliases: [verify_ssl]
      security_protocol:
        type: str
        choices: ['ssl-with-validation', 'ssl-with-validation-custom-ca', 'ssl-without-validation', 'non-ssl']
        description:
          - TODO needs documentation.
      certificate_authority:
        type: str
        description:
          - TODO needs documentation.
      password:
        type: str
        description:
          - TODO needs documentation.
      path:
        type: str
        description:
          - TODO needs documentation.
      project:
        type: str
        description:
          - TODO needs documentation.
      role:
        type: str
        description:
          - TODO needs documentation.
      subscription:
        type: str
        description:
          - TODO needs documentation.
      uid_ems:
        type: str
        description:
          - TODO needs documentation.
      port:
        type: int
        description:
          - TODO needs documentation.
"""

EXAMPLES = r"""
- name: Create a new provider in ManageIQ ('Hawkular' metrics)
  community.general.manageiq_provider:
    name: 'EngLab'
    type: 'OpenShift'
    state: 'present'
    provider:
      auth_key: 'topSecret'
      hostname: 'example.com'
      port: 8443
      validate_certs: true
      security_protocol: 'ssl-with-validation-custom-ca'
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    metrics:
      auth_key: 'topSecret'
      role: 'hawkular'
      hostname: 'example.com'
      port: 443
      validate_certs: true
      security_protocol: 'ssl-with-validation-custom-ca'
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    manageiq_connection:
      url: 'https://127.0.0.1:80'
      username: 'admin'
      password: 'password'
      validate_certs: true


- name: Update an existing provider named 'EngLab' (defaults to 'Prometheus' metrics)
  community.general.manageiq_provider:
    name: 'EngLab'
    type: 'Openshift'
    state: 'present'
    provider:
      auth_key: 'topSecret'
      hostname: 'next.example.com'
      port: 8443
      validate_certs: true
      security_protocol: 'ssl-with-validation-custom-ca'
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    metrics:
      auth_key: 'topSecret'
      hostname: 'next.example.com'
      port: 443
      validate_certs: true
      security_protocol: 'ssl-with-validation-custom-ca'
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    manageiq_connection:
      url: 'https://127.0.0.1'
      username: 'admin'
      password: 'password'
      validate_certs: true


- name: Delete a provider in ManageIQ
  community.general.manageiq_provider:
    name: 'EngLab'
    type: 'Openshift'
    state: 'absent'
    manageiq_connection:
      url: 'https://127.0.0.1'
      username: 'admin'
      password: 'password'
      validate_certs: true


- name: Create a new Amazon provider in ManageIQ using token authentication
  community.general.manageiq_provider:
    name: 'EngAmazon'
    type: 'Amazon'
    state: 'present'
    provider:
      hostname: 'amazon.example.com'
      userid: 'hello'
      password: 'world'
    manageiq_connection:
      url: 'https://127.0.0.1'
      token: 'VeryLongToken'
      validate_certs: true


- name: Create a new oVirt provider in ManageIQ
  community.general.manageiq_provider:
    name: 'RHEV'
    type: 'oVirt'
    state: 'present'
    provider:
      hostname: 'rhev01.example.com'
      userid: 'admin@internal'
      password: 'password'
      validate_certs: true
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    metrics:
      hostname: 'metrics.example.com'
      path: 'ovirt_engine_history'
      userid: 'user_id_metrics'
      password: 'password_metrics'
      validate_certs: true
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    manageiq_connection:
      url: 'https://127.0.0.1'
      username: 'admin'
      password: 'password'
      validate_certs: true

- name: Create a new VMware provider in ManageIQ
  community.general.manageiq_provider:
    name: 'EngVMware'
    type: 'VMware'
    state: 'present'
    provider:
      hostname: 'vcenter.example.com'
      host_default_vnc_port_start: 5800
      host_default_vnc_port_end: 5801
      userid: 'root'
      password: 'password'
    manageiq_connection:
      url: 'https://127.0.0.1'
      token: 'VeryLongToken'
      validate_certs: true

- name: Create a new Azure provider in ManageIQ
  community.general.manageiq_provider:
    name: 'EngAzure'
    type: 'Azure'
    provider_region: 'northeurope'
    subscription: 'e272bd74-f661-484f-b223-88dd128a4049'
    azure_tenant_id: 'e272bd74-f661-484f-b223-88dd128a4048'
    state: 'present'
    provider:
      hostname: 'azure.example.com'
      userid: 'e272bd74-f661-484f-b223-88dd128a4049'
      password: 'password'
    manageiq_connection:
      url: 'https://cf-6af0.rhpds.opentlc.com'
      username: 'admin'
      password: 'password'
      validate_certs: true

- name: Create a new OpenStack Director provider in ManageIQ with rsa keypair
  community.general.manageiq_provider:
    name: 'EngDirector'
    type: 'Director'
    api_version: 'v3'
    state: 'present'
    provider:
      hostname: 'director.example.com'
      userid: 'admin'
      password: 'password'
      security_protocol: 'ssl-with-validation'
      validate_certs: 'true'
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    ssh_keypair:
      hostname: director.example.com
      userid: heat-admin
      auth_key: 'SecretSSHPrivateKey'

- name: Create a new OpenStack provider in ManageIQ with amqp metrics
  community.general.manageiq_provider:
    name: 'EngOpenStack'
    type: 'OpenStack'
    api_version: 'v3'
    state: 'present'
    provider_region: 'europe'
    tenant_mapping_enabled: 'False'
    keystone_v3_domain_id: 'mydomain'
    provider:
      hostname: 'openstack.example.com'
      userid: 'admin'
      password: 'password'
      security_protocol: 'ssl-with-validation'
      validate_certs: 'true'
      certificate_authority: |
        -----BEGIN CERTIFICATE-----
        FAKECERTsdKgAwI...
        -----END CERTIFICATE-----
    metrics:
      role: amqp
      hostname: 'amqp.example.com'
      security_protocol: 'non-ssl'
      port: 5666
      userid: admin
      password: password


- name: Create a new GCE provider in ManageIQ
  community.general.manageiq_provider:
    name: 'EngGoogle'
    type: 'GCE'
    provider_region: 'europe-west1'
    project: 'project1'
    state: 'present'
    provider:
      hostname: 'gce.example.com'
      auth_key: 'google_json_key'
      validate_certs: 'false'
"""

RETURN = r"""
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._manageiq import ManageIQ, manageiq_argument_spec


def supported_providers():
    return dict(
        Openshift=dict(
            class_name="ManageIQ::Providers::Openshift::ContainerManager",
            authtype="bearer",
            default_role="default",
            metrics_role="prometheus",
            alerts_role="prometheus_alerts",
        ),
        Amazon=dict(
            class_name="ManageIQ::Providers::Amazon::CloudManager",
        ),
        oVirt=dict(
            class_name="ManageIQ::Providers::Redhat::InfraManager",
            default_role="default",
            metrics_role="metrics",
        ),
        VMware=dict(
            class_name="ManageIQ::Providers::Vmware::InfraManager",
        ),
        Azure=dict(
            class_name="ManageIQ::Providers::Azure::CloudManager",
        ),
        Director=dict(class_name="ManageIQ::Providers::Openstack::InfraManager", ssh_keypair_role="ssh_keypair"),
        OpenStack=dict(
            class_name="ManageIQ::Providers::Openstack::CloudManager",
        ),
        GCE=dict(
            class_name="ManageIQ::Providers::Google::CloudManager",
        ),
    )


def endpoint_list_spec():
    return dict(
        provider=dict(type="dict", options=endpoint_argument_spec()),
        metrics=dict(type="dict", options=endpoint_argument_spec()),
        alerts=dict(type="dict", options=endpoint_argument_spec()),
        ssh_keypair=dict(type="dict", options=endpoint_argument_spec(), no_log=False),
    )


def endpoint_argument_spec():
    return dict(
        role=dict(),
        hostname=dict(required=True),
        port=dict(type="int"),
        validate_certs=dict(default=True, type="bool", aliases=["verify_ssl"]),
        certificate_authority=dict(),
        security_protocol=dict(
            choices=[
                "ssl-with-validation",
                "ssl-with-validation-custom-ca",
                "ssl-without-validation",
                "non-ssl",
            ],
        ),
        userid=dict(),
        password=dict(no_log=True),
        auth_key=dict(no_log=True),
        subscription=dict(no_log=True),
        project=dict(),
        uid_ems=dict(),
        path=dict(),
    )


def delete_nulls(h):
    """Remove null entries from a hash

    Returns:
        a hash without nulls
    """
    if isinstance(h, list):
        return [delete_nulls(i) for i in h]
    if isinstance(h, dict):
        return {k: delete_nulls(v) for k, v in h.items() if v is not None}

    return h


class ManageIQProvider:
    """
    Object to execute provider management operations in manageiq.
    """

    def __init__(self, manageiq):
        self.manageiq = manageiq

        self.module = self.manageiq.module
        self.api_url = self.manageiq.api_url
        self.client = self.manageiq.client

    def class_name_to_type(self, class_name):
        """Convert class_name to type

        Returns:
            the type
        """
        out = [k for k, v in supported_providers().items() if v["class_name"] == class_name]
        if len(out) == 1:
            return out[0]

        return None

    def zone_id(self, name):
        """Search for zone id by zone name.

        Returns:
            the zone id, or send a module Fail signal if zone not found.
        """
        zone = self.manageiq.find_collection_resource_by("zones", name=name)
        if not zone:  # zone doesn't exist
            self.module.fail_json(msg=f"zone {name} does not exist in manageiq")

        return zone["id"]

    def provider(self, name):
        """Search for provider object by name.

        Returns:
            the provider, or None if provider not found.
        """
        return self.manageiq.find_collection_resource_by("providers", name=name)

    def build_connection_configurations(self, provider_type, endpoints):
        """Build "connection_configurations" objects from
        requested endpoints provided by user

        Returns:
            the user requested provider endpoints list
        """
        connection_configurations = []
        endpoint_keys = endpoint_list_spec().keys()
        provider_defaults = supported_providers().get(provider_type, {})

        # get endpoint defaults
        endpoint = endpoints.get("provider")
        default_auth_key = endpoint.get("auth_key")

        # build a connection_configuration object for each endpoint
        for endpoint_key in endpoint_keys:
            endpoint = endpoints.get(endpoint_key)
            if endpoint:
                # get role and authtype
                role = endpoint.get("role") or provider_defaults.get(f"{endpoint_key}_role", "default")
                if role == "default":
                    authtype = provider_defaults.get("authtype") or role
                else:
                    authtype = role

                # set a connection_configuration
                connection_configurations.append(
                    {
                        "endpoint": {
                            "role": role,
                            "hostname": endpoint.get("hostname"),
                            "port": endpoint.get("port"),
                            "verify_ssl": [0, 1][endpoint.get("validate_certs", True)],
                            "security_protocol": endpoint.get("security_protocol"),
                            "certificate_authority": endpoint.get("certificate_authority"),
                            "path": endpoint.get("path"),
                        },
                        "authentication": {
                            "authtype": authtype,
                            "userid": endpoint.get("userid"),
                            "password": endpoint.get("password"),
                            "auth_key": endpoint.get("auth_key") or default_auth_key,
                        },
                    }
                )

        return connection_configurations

    def delete_provider(self, provider):
        """Deletes a provider from manageiq.

        Returns:
            a short message describing the operation executed.
        """
        try:
            url = f"{self.api_url}/providers/{provider['id']}"
            result = self.client.post(url, action="delete")
        except Exception as e:
            self.module.fail_json(msg=f"failed to delete provider {provider['name']}: {e}")

        return dict(changed=True, msg=result["message"])

    def edit_provider(
        self,
        provider,
        name,
        provider_type,
        endpoints,
        zone_id,
        provider_region,
        host_default_vnc_port_start,
        host_default_vnc_port_end,
        subscription,
        project,
        uid_ems,
        tenant_mapping_enabled,
        api_version,
    ):
        """Edit a provider from manageiq.

        Returns:
            a short message describing the operation executed.
        """
        url = f"{self.api_url}/providers/{provider['id']}"

        resource = dict(
            name=name,
            zone={"id": zone_id},
            provider_region=provider_region,
            connection_configurations=endpoints,
            host_default_vnc_port_start=host_default_vnc_port_start,
            host_default_vnc_port_end=host_default_vnc_port_end,
            subscription=subscription,
            project=project,
            uid_ems=uid_ems,
            tenant_mapping_enabled=tenant_mapping_enabled,
            api_version=api_version,
        )

        # NOTE: we do not check for diff's between requested and current
        #       provider, we always submit endpoints with password or auth_keys,
        #       since we can not compare with current password or auth_key,
        #       every edit request is sent to ManageIQ API without comparing
        #       it to current state.

        # clean nulls, we do not send nulls to the api
        resource = delete_nulls(resource)

        # try to update provider
        try:
            result = self.client.post(url, action="edit", resource=resource)
        except Exception as e:
            self.module.fail_json(msg=f"failed to update provider {provider['name']}: {e}")

        return dict(changed=True, msg=f"successfully updated the provider {provider['name']}: {result}")

    def create_provider(
        self,
        name,
        provider_type,
        endpoints,
        zone_id,
        provider_region,
        host_default_vnc_port_start,
        host_default_vnc_port_end,
        subscription,
        project,
        uid_ems,
        tenant_mapping_enabled,
        api_version,
    ):
        """Creates the provider in manageiq.

        Returns:
            a short message describing the operation executed.
        """
        resource = dict(
            name=name,
            zone={"id": zone_id},
            provider_region=provider_region,
            host_default_vnc_port_start=host_default_vnc_port_start,
            host_default_vnc_port_end=host_default_vnc_port_end,
            subscription=subscription,
            project=project,
            uid_ems=uid_ems,
            tenant_mapping_enabled=tenant_mapping_enabled,
            api_version=api_version,
            connection_configurations=endpoints,
        )

        # clean nulls, we do not send nulls to the api
        resource = delete_nulls(resource)

        # try to create a new provider
        try:
            url = f"{self.api_url}/providers"
            result = self.client.post(url, type=supported_providers()[provider_type]["class_name"], **resource)
        except Exception as e:
            self.module.fail_json(msg=f"failed to create provider {name}: {e}")

        return dict(changed=True, msg=f"successfully created the provider {name}: {result['results']}")

    def refresh(self, provider, name):
        """Trigger provider refresh.

        Returns:
            a short message describing the operation executed.
        """
        try:
            url = f"{self.api_url}/providers/{provider['id']}"
            self.client.post(url, action="refresh")
        except Exception as e:
            self.module.fail_json(msg=f"failed to refresh provider {name}: {e}")

        return dict(changed=True, msg=f"refreshing provider {name}")


def main():
    zone_id = None
    endpoints = []
    argument_spec = dict(
        state=dict(choices=["absent", "present", "refresh"], default="present"),
        name=dict(required=True),
        zone=dict(default="default"),
        provider_region=dict(),
        host_default_vnc_port_start=dict(),
        host_default_vnc_port_end=dict(),
        subscription=dict(),
        project=dict(),
        azure_tenant_id=dict(aliases=["keystone_v3_domain_id"]),
        tenant_mapping_enabled=dict(default=False, type="bool"),
        api_version=dict(choices=["v2", "v3"]),
        type=dict(choices=list(supported_providers().keys())),
    )
    # add the manageiq connection arguments to the arguments
    argument_spec.update(manageiq_argument_spec())
    # add the endpoint arguments to the arguments
    argument_spec.update(endpoint_list_spec())

    module = AnsibleModule(
        argument_spec=argument_spec,
        required_if=[("state", "present", ["provider"]), ("state", "refresh", ["name"])],
        required_together=[["host_default_vnc_port_start", "host_default_vnc_port_end"]],
    )

    name = module.params["name"]
    zone_name = module.params["zone"]
    provider_type = module.params["type"]
    raw_endpoints = module.params
    provider_region = module.params["provider_region"]
    host_default_vnc_port_start = module.params["host_default_vnc_port_start"]
    host_default_vnc_port_end = module.params["host_default_vnc_port_end"]
    subscription = module.params["subscription"]
    uid_ems = module.params["azure_tenant_id"]
    project = module.params["project"]
    tenant_mapping_enabled = module.params["tenant_mapping_enabled"]
    api_version = module.params["api_version"]
    state = module.params["state"]

    manageiq = ManageIQ(module)
    manageiq_provider = ManageIQProvider(manageiq)

    provider = manageiq_provider.provider(name)

    # provider should not exist
    if state == "absent":
        # if we have a provider, delete it
        if provider:
            res_args = manageiq_provider.delete_provider(provider)
        # if we do not have a provider, nothing to do
        else:
            res_args = dict(changed=False, msg=f"provider {name}: does not exist in manageiq")

    # provider should exist
    if state == "present":
        # get data user did not explicitly give
        if zone_name:
            zone_id = manageiq_provider.zone_id(zone_name)

        # if we do not have a provider_type, use the current provider_type
        if provider and not provider_type:
            provider_type = manageiq_provider.class_name_to_type(provider["type"])

        # check supported_providers types
        if not provider_type:
            manageiq_provider.module.fail_json(msg="missing required argument: provider_type")

        # check supported_providers types
        if provider_type not in supported_providers().keys():
            manageiq_provider.module.fail_json(msg=f"provider_type {provider_type} is not supported")

        # build "connection_configurations" objects from user requested endpoints
        # "provider" is a required endpoint, if we have it, we have endpoints
        if raw_endpoints.get("provider"):
            endpoints = manageiq_provider.build_connection_configurations(provider_type, raw_endpoints)

        # if we have a provider, edit it
        if provider:
            res_args = manageiq_provider.edit_provider(
                provider,
                name,
                provider_type,
                endpoints,
                zone_id,
                provider_region,
                host_default_vnc_port_start,
                host_default_vnc_port_end,
                subscription,
                project,
                uid_ems,
                tenant_mapping_enabled,
                api_version,
            )
        # if we do not have a provider, create it
        else:
            res_args = manageiq_provider.create_provider(
                name,
                provider_type,
                endpoints,
                zone_id,
                provider_region,
                host_default_vnc_port_start,
                host_default_vnc_port_end,
                subscription,
                project,
                uid_ems,
                tenant_mapping_enabled,
                api_version,
            )

    # refresh provider (trigger sync)
    if state == "refresh":
        if provider:
            res_args = manageiq_provider.refresh(provider, name)
        else:
            res_args = dict(changed=False, msg=f"provider {name}: does not exist in manageiq")

    module.exit_json(**res_args)


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2023, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_compose_v2_exec

short_description: Run command in a container of a Compose service

version_added: 3.13.0

description:
  - Uses Docker Compose to run a command in a service's container.
  - This can be used to run one-off commands in an existing service's container, and encapsulates C(docker compose exec).
extends_documentation_fragment:
  - community.docker._compose_v2
  - community.docker._compose_v2.minimum_version
  - community.docker._docker.cli_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  idempotent:
    support: N/A
    details:
      - Whether the executed command is idempotent depends on the command.

options:
  service:
    description:
      - The service to run the command in.
    type: str
    required: true
  index:
    description:
      - The index of the container to run the command in if the service has multiple replicas.
    type: int
  argv:
    type: list
    elements: str
    description:
      - The command to execute.
      - Since this is a list of arguments, no quoting is needed.
      - Exactly one of O(argv) or O(command) must be specified.
  command:
    type: str
    description:
      - The command to execute.
      - Exactly one of O(argv) or O(command) must be specified.
  chdir:
    type: str
    description:
      - The directory to run the command in.
  detach:
    description:
      - Whether to run the command synchronously (O(detach=false), default) or asynchronously (O(detach=true)).
      - If set to V(true), O(stdin) cannot be provided, and the return values RV(stdout), RV(stderr), and RV(rc) are not returned.
    type: bool
    default: false
  user:
    type: str
    description:
      - If specified, the user to execute this command with.
  stdin:
    type: str
    description:
      - Set the stdin of the command directly to the specified value.
      - Can only be used if O(detach=false).
  stdin_add_newline:
    type: bool
    default: true
    description:
      - If set to V(true), appends a newline to O(stdin).
  strip_empty_ends:
    type: bool
    default: true
    description:
      - Strip empty lines from the end of stdout/stderr in result.
  privileged:
    type: bool
    default: false
    description:
      - Whether to give extended privileges to the process.
  tty:
    type: bool
    default: true
    description:
      - Whether to allocate a TTY.
  env:
    description:
      - Dictionary of environment variables with their respective values to be passed to the command ran inside the container.
      - Values which might be parsed as numbers, booleans or other types by the YAML parser must be quoted (for example V("true"))
        in order to avoid data loss.
      - Please note that if you are passing values in with Jinja2 templates, like V("{{ value }}"), you need to add V(| string)
        to prevent Ansible to convert strings such as V("true") back to booleans. The correct way is to use V("{{ value |
        string }}").
    type: dict

author:
  - Felix Fontein (@felixfontein)

seealso:
  - module: community.docker.docker_compose_v2

notes:
  - If you need to evaluate environment variables of the container in O(command) or O(argv), you need to pass the command
    through a shell, like O(command=/bin/sh -c "echo $ENV_VARIABLE"). The same needs to be done in case you want to use glob patterns
    or other shell features such as redirects.
"""

EXAMPLES = r"""
---
- name: Run a simple command (command)
  community.docker.docker_compose_v2_exec:
    service: foo
    command: /bin/bash -c "ls -lah"
    chdir: /root
  register: result

- name: Print stdout
  ansible.builtin.debug:
    var: result.stdout

- name: Run a simple command (argv)
  community.docker.docker_compose_v2_exec:
    service: foo
    argv:
      - /bin/bash
      - "-c"
      - "ls -lah > /dev/stderr"
    chdir: /root
  register: result

- name: Print stderr lines
  ansible.builtin.debug:
    var: result.stderr_lines
"""

RETURN = r"""
stdout:
  type: str
  returned: success and O(detach=false)
  description:
    - The standard output of the container command.
stderr:
  type: str
  returned: success and O(detach=false)
  description:
    - The standard error output of the container command.
rc:
  type: int
  returned: success and O(detach=false)
  sample: 0
  description:
    - The exit code of the command.
"""

import shlex
import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_text

from ansible_collections.community.docker.plugins.module_utils._common_cli import (
    AnsibleModuleDockerClient,
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._compose_v2 import (
    BaseComposeManager,
    common_compose_argspec_ex,
)


class ExecManager(BaseComposeManager):
    def __init__(self, client: AnsibleModuleDockerClient) -> None:
        super().__init__(client)
        parameters = self.client.module.params

        self.service: str = parameters["service"]
        self.index: int | None = parameters["index"]
        self.chdir: str | None = parameters["chdir"]
        self.detach: bool = parameters["detach"]
        self.user: str | None = parameters["user"]
        self.stdin: str | None = parameters["stdin"]
        self.strip_empty_ends: bool = parameters["strip_empty_ends"]
        self.privileged: bool = parameters["privileged"]
        self.tty: bool = parameters["tty"]
        self.env: dict[str, t.Any] = parameters["env"]

        self.argv: list[str]
        if parameters["command"] is not None:
            self.argv = shlex.split(parameters["command"])
        else:
            self.argv = parameters["argv"]

        if self.detach and self.stdin is not None:
            self.fail("If detach=true, stdin cannot be provided.")

        stdin_add_newline: bool = parameters["stdin_add_newline"]
        if self.stdin is not None and stdin_add_newline:
            self.stdin += "\n"

        if self.env is not None:
            for name, value in self.env.items():
                if not isinstance(value, str):
                    self.fail(
                        "Non-string value found for env option. Ambiguous env options must be "
                        "wrapped in quotes to avoid them being interpreted when directly specified "
                        "in YAML, or explicitly converted to strings when the option is templated. "
                        f"Key: {name}"
                    )

    def get_exec_cmd(self, dry_run: bool) -> list[str]:
        args = self.get_base_args(plain_progress=True) + ["exec"]
        if self.index is not None:
            args.extend(["--index", str(self.index)])
        if self.chdir is not None:
            args.extend(["--workdir", self.chdir])
        if self.detach:
            args.extend(["--detach"])
        if self.user is not None:
            args.extend(["--user", self.user])
        if self.privileged:
            args.append("--privileged")
        if not self.tty:
            args.append("--no-TTY")
        if self.env:
            for name, value in list(self.env.items()):
                args.append("--env")
                args.append(f"{name}={value}")
        args.append("--")
        args.append(self.service)
        args.extend(self.argv)
        return args

    def run(self) -> dict[str, t.Any]:
        args = self.get_exec_cmd(self.check_mode)
        kwargs: dict[str, t.Any] = {
            "cwd": self.project_src,
        }
        if self.stdin is not None:
            kwargs["data"] = self.stdin.encode("utf-8")
        if self.detach:
            kwargs["check_rc"] = True
        rc, stdout_b, stderr_b = self.client.call_cli(*args, **kwargs)
        if self.detach:
            return {}
        stdout = to_text(stdout_b)
        stderr = to_text(stderr_b)
        if self.strip_empty_ends:
            stdout = stdout.rstrip("\r\n")
            stderr = stderr.rstrip("\r\n")
        return {
            "changed": True,
            "rc": rc,
            "stdout": stdout,
            "stderr": stderr,
        }


def main() -> None:
    argument_spec = {
        "service": {"type": "str", "required": True},
        "index": {"type": "int"},
        "argv": {"type": "list", "elements": "str"},
        "command": {"type": "str"},
        "chdir": {"type": "str"},
        "detach": {"type": "bool", "default": False},
        "user": {"type": "str"},
        "stdin": {"type": "str"},
        "stdin_add_newline": {"type": "bool", "default": True},
        "strip_empty_ends": {"type": "bool", "default": True},
        "privileged": {"type": "bool", "default": False},
        "tty": {"type": "bool", "default": True},
        "env": {"type": "dict"},
    }
    argspec_ex = common_compose_argspec_ex()
    argument_spec.update(argspec_ex.pop("argspec"))

    client = AnsibleModuleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=False,
        needs_api_version=False,
        **argspec_ex,
    )

    try:
        manager = ExecManager(client)
        result = manager.run()
        manager.cleanup()
        client.module.exit_json(**result)
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2018, REY Remi
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: secretsmanager_secret
version_added: 1.0.0
short_description: Manage secrets stored in AWS Secrets Manager
description:
  - Create, update, and delete secrets stored in AWS Secrets Manager.
  - Prior to release 5.0.0 this module was called C(community.aws.aws_secret).
    The usage did not change.
author:
  - "REY Remi (@rrey)"
options:
  name:
    description:
    - Friendly name for the secret you are creating.
    required: true
    type: str
  state:
    description:
    - Whether the secret should be exist or not.
    default: 'present'
    choices: ['present', 'absent']
    type: str
  overwrite:
    description:
    - Whether to overwrite an existing secret with the same name.
    - If set to C(True), an existing secret with the same I(name) will be overwritten.
    - If set to C(False), a secret with the given I(name) will only be created if none exists.
    type: bool
    default: True
    version_added: 5.3.0
  recovery_window:
    description:
    - Only used if state is absent.
    - Specifies the number of days that Secrets Manager waits before it can delete the secret.
    - If set to 0, the deletion is forced without recovery.
    default: 30
    type: int
  description:
    description:
    - Specifies a user-provided description of the secret.
    type: str
    default: ''
  replica:
    description:
    - Specifies a list of regions and kms_key_ids (optional) to replicate the secret to
    type: list
    elements: dict
    version_added: 5.3.0
    suboptions:
      region:
        description:
          - Region to replicate secret to.
        type: str
        required: true
      kms_key_id:
        description:
          - Specifies the ARN or alias of the AWS KMS customer master key (CMK) in the
            destination region to be used (alias/aws/secretsmanager is assumed if not specified)
        type: str
        required: false
  kms_key_id:
    description:
    - Specifies the ARN or alias of the AWS KMS customer master key (CMK) to be
      used to encrypt the I(secret) values in the versions stored in this secret.
    type: str
  secret_type:
    description:
    - Specifies the type of data that you want to encrypt.
    choices: ['binary', 'string']
    default: 'string'
    type: str
  secret:
    description:
    - Specifies string or binary data that you want to encrypt and store in the new version of the secret.
    - Mutually exclusive with the I(json_secret) option.
    default: ""
    type: str
  json_secret:
    description:
    - Specifies JSON-formatted data that you want to encrypt and store in the new version of the
      secret.
    - Mutually exclusive with the I(secret) option.
    type: json
    version_added: 4.1.0
  resource_policy:
    description:
    - Specifies JSON-formatted resource policy to attach to the secret. Useful when granting cross-account access
       to secrets.
    required: false
    type: json
    version_added: 3.1.0
  rotation_lambda:
    description:
    - Specifies the ARN of the Lambda function that can rotate the secret.
    type: str
  rotation_interval:
    description:
    - Specifies the number of days between automatic scheduled rotations of the secret.
    default: 30
    type: int
notes:
  - Support for I(purge_tags) was added in release 4.0.0.
extends_documentation_fragment:
  - amazon.aws.region.modules
  - amazon.aws.common.modules
  - amazon.aws.tags
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Add string to AWS Secrets Manager
  community.aws.secretsmanager_secret:
    name: 'test_secret_string'
    state: present
    secret_type: 'string'
    secret: "{{ super_secret_string }}"

- name: Add a secret with resource policy attached
  community.aws.secretsmanager_secret:
    name: 'test_secret_string'
    state: present
    secret_type: 'string'
    secret: "{{ super_secret_string }}"
    resource_policy: "{{ lookup('template', 'templates/resource_policy.json.j2', convert_data=False) | string }}"

- name: remove string from AWS Secrets Manager
  community.aws.secretsmanager_secret:
    name: 'test_secret_string'
    state: absent
    secret_type: 'string'
    secret: "{{ super_secret_string }}"

- name: Only create a new secret, but do not update if alredy exists by name
  community.aws.secretsmanager_secret:
    name: 'random_string'
    state: present
    secret_type: 'string'
    secret: "{{ lookup('community.general.random_string', length=16, special=false) }}"
    overwrite: false
"""

RETURN = r"""
secret:
  description: The secret information
  returned: always
  type: complex
  contains:
    arn:
      description: The ARN of the secret.
      returned: always
      type: str
      sample: arn:aws:secretsmanager:eu-west-1:xxxxxxxxxx:secret:xxxxxxxxxxx
    description:
      description: A description of the secret.
      returned: when the secret has a description
      type: str
      sample: An example description
    last_accessed_date:
      description: The date the secret was last accessed.
      returned: always
      type: str
      sample: '2018-11-20T01:00:00+01:00'
    last_changed_date:
      description: The date the secret was last modified.
      returned: always
      type: str
      sample: '2018-11-20T12:16:38.433000+01:00'
    name:
      description: The secret name.
      returned: always
      type: str
      sample: my_secret
    rotation_enabled:
      description: The secret rotation status.
      returned: always
      type: bool
      sample: false
    version_ids_to_stages:
      description: Provide the secret version ids and the associated secret stage.
      returned: always
      type: dict
      sample: { "dc1ed59b-6d8e-4450-8b41-536dfe4600a9": [ "AWSCURRENT" ] }
    tags:
      description:
        - A list of dictionaries representing the tags associated with the secret in the standard boto3 format.
      returned: when the secret has tags
      type: list
      elements: dict
      contains:
        key:
          description: The name or key of the tag.
          type: str
          example: MyTag
          returned: success
        value:
          description: The value of the tag.
          type: str
          example: Some value.
          returned: success
    tags_dict:
      description: A dictionary representing the tags associated with the secret.
      type: dict
      returned: when the secret has tags
      example: {'MyTagName': 'Some Value'}
      version_added: 4.0.0
"""

import json
from traceback import format_exc

try:
    from botocore.exceptions import BotoCoreError
    from botocore.exceptions import ClientError
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils._text import to_bytes
from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict
from ansible.module_utils.common.dict_transformations import snake_dict_to_camel_dict

from ansible_collections.amazon.aws.plugins.module_utils.policy import compare_policies
from ansible_collections.amazon.aws.plugins.module_utils.tagging import ansible_dict_to_boto3_tag_list
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict
from ansible_collections.amazon.aws.plugins.module_utils.tagging import compare_aws_tags

from ansible_collections.community.aws.plugins.module_utils.modules import AnsibleCommunityAWSModule as AnsibleAWSModule


class Secret(object):
    """An object representation of the Secret described by the self.module args"""

    def __init__(
        self,
        name,
        secret_type,
        secret,
        resource_policy=None,
        description="",
        kms_key_id=None,
        tags=None,
        lambda_arn=None,
        rotation_interval=None,
        replica_regions=None,
    ):
        self.name = name
        self.description = description
        self.replica_regions = replica_regions
        self.kms_key_id = kms_key_id
        if secret_type == "binary":
            self.secret_type = "SecretBinary"
        else:
            self.secret_type = "SecretString"
        self.secret = secret
        self.resource_policy = resource_policy
        self.tags = tags or {}
        self.rotation_enabled = False
        if lambda_arn:
            self.rotation_enabled = True
            self.rotation_lambda_arn = lambda_arn
            self.rotation_rules = {"AutomaticallyAfterDays": int(rotation_interval)}

    @property
    def create_args(self):
        args = {"Name": self.name}
        if self.description:
            args["Description"] = self.description
        if self.kms_key_id:
            args["KmsKeyId"] = self.kms_key_id
        if self.replica_regions:
            add_replica_regions = []
            for replica in self.replica_regions:
                if replica["kms_key_id"]:
                    add_replica_regions.append({"Region": replica["region"], "KmsKeyId": replica["kms_key_id"]})
                else:
                    add_replica_regions.append({"Region": replica["region"]})
            args["AddReplicaRegions"] = add_replica_regions
        if self.tags:
            args["Tags"] = ansible_dict_to_boto3_tag_list(self.tags)
        args[self.secret_type] = self.secret
        return args

    @property
    def update_args(self):
        args = {"SecretId": self.name}
        if self.description:
            args["Description"] = self.description
        if self.kms_key_id:
            args["KmsKeyId"] = self.kms_key_id
        args[self.secret_type] = self.secret
        return args

    @property
    def secret_resource_policy_args(self):
        args = {"SecretId": self.name}
        if self.resource_policy:
            args["ResourcePolicy"] = self.resource_policy
        return args

    @property
    def boto3_tags(self):
        return ansible_dict_to_boto3_tag_list(self.Tags)

    def as_dict(self):
        result = self.__dict__
        result.pop("tags")
        return snake_dict_to_camel_dict(result)


class SecretsManagerInterface(object):
    """An interface with SecretsManager"""

    def __init__(self, module):
        self.module = module
        self.client = self.module.client("secretsmanager")

    def get_secret(self, name):
        try:
            secret = self.client.describe_secret(SecretId=name)
        except self.client.exceptions.ResourceNotFoundException:
            secret = None
        except Exception as e:
            self.module.fail_json_aws(e, msg="Failed to describe secret")
        return secret

    def get_resource_policy(self, name):
        try:
            resource_policy = self.client.get_resource_policy(SecretId=name)
        except self.client.exceptions.ResourceNotFoundException:
            resource_policy = None
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to get secret resource policy")
        return resource_policy

    def create_secret(self, secret):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            created_secret = self.client.create_secret(**secret.create_args)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to create secret")

        if secret.rotation_enabled:
            response = self.update_rotation(secret)
            created_secret["VersionId"] = response.get("VersionId")
        return created_secret

    def update_secret(self, secret):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            response = self.client.update_secret(**secret.update_args)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to update secret")
        return response

    def put_resource_policy(self, secret):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            json.loads(secret.secret_resource_policy_args.get("ResourcePolicy"))
        except (TypeError, ValueError) as e:
            self.module.fail_json(msg=f"Failed to parse resource policy as JSON: {str(e)}", exception=format_exc())

        try:
            response = self.client.put_resource_policy(**secret.secret_resource_policy_args)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to update secret resource policy")
        return response

    def remove_replication(self, name, regions):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            response = self.client.remove_regions_from_replication(SecretId=name, RemoveReplicaRegions=regions)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to replicate secret")
        return response

    def replicate_secret(self, name, regions):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            replica_regions = []
            for replica in regions:
                if replica["kms_key_id"]:
                    replica_regions.append({"Region": replica["region"], "KmsKeyId": replica["kms_key_id"]})
                else:
                    replica_regions.append({"Region": replica["region"]})
            response = self.client.replicate_secret_to_regions(SecretId=name, AddReplicaRegions=replica_regions)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to replicate secret")
        return response

    def restore_secret(self, name):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            response = self.client.restore_secret(SecretId=name)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to restore secret")
        return response

    def delete_secret(self, name, recovery_window):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            if recovery_window == 0:
                response = self.client.delete_secret(SecretId=name, ForceDeleteWithoutRecovery=True)
            else:
                response = self.client.delete_secret(SecretId=name, RecoveryWindowInDays=recovery_window)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to delete secret")
        return response

    def delete_resource_policy(self, name):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            response = self.client.delete_resource_policy(SecretId=name)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to delete secret resource policy")
        return response

    def update_rotation(self, secret):
        if secret.rotation_enabled:
            try:
                response = self.client.rotate_secret(
                    SecretId=secret.name,
                    RotationLambdaARN=secret.rotation_lambda_arn,
                    RotationRules=secret.rotation_rules,
                )
            except (BotoCoreError, ClientError) as e:
                self.module.fail_json_aws(e, msg="Failed to rotate secret secret")
        else:
            try:
                response = self.client.cancel_rotate_secret(SecretId=secret.name)
            except (BotoCoreError, ClientError) as e:
                self.module.fail_json_aws(e, msg="Failed to cancel rotation")
        return response

    def tag_secret(self, secret_name, tags):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            self.client.tag_resource(SecretId=secret_name, Tags=tags)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to add tag(s) to secret")

    def untag_secret(self, secret_name, tag_keys):
        if self.module.check_mode:
            self.module.exit_json(changed=True)
        try:
            self.client.untag_resource(SecretId=secret_name, TagKeys=tag_keys)
        except (BotoCoreError, ClientError) as e:
            self.module.fail_json_aws(e, msg="Failed to remove tag(s) from secret")

    def secrets_match(self, desired_secret, current_secret):
        """Compare secrets except tags and rotation

        Args:
            desired_secret: camel dict representation of the desired secret state.
            current_secret: secret reference as returned by the secretsmanager api.

        Returns: bool
        """
        if desired_secret.description != current_secret.get("Description", ""):
            return False
        if desired_secret.kms_key_id != current_secret.get("KmsKeyId"):
            return False
        current_secret_value = self.client.get_secret_value(SecretId=current_secret.get("Name"))
        if desired_secret.secret_type == "SecretBinary":
            desired_value = to_bytes(desired_secret.secret)
        else:
            desired_value = desired_secret.secret
        if desired_value != current_secret_value.get(desired_secret.secret_type):
            return False
        return True


def rotation_match(desired_secret, current_secret):
    """Compare secrets rotation configuration

    Args:
        desired_secret: camel dict representation of the desired secret state.
        current_secret: secret reference as returned by the secretsmanager api.

    Returns: bool
    """
    if desired_secret.rotation_enabled != current_secret.get("RotationEnabled", False):
        return False
    if desired_secret.rotation_enabled:
        if desired_secret.rotation_lambda_arn != current_secret.get("RotationLambdaARN"):
            return False
        if desired_secret.rotation_rules != current_secret.get("RotationRules"):
            return False
    return True


def compare_regions(desired_secret, current_secret):
    """Compare secrets replication configuration

    Args:
        desired_secret: camel dict representation of the desired secret state.
        current_secret: secret reference as returned by the secretsmanager api.

    Returns: bool
    """
    regions_to_set_replication = []
    regions_to_remove_replication = []

    if desired_secret.replica_regions is None:
        return regions_to_set_replication, regions_to_remove_replication

    if desired_secret.replica_regions:
        regions_to_set_replication = desired_secret.replica_regions

    for current_secret_region in current_secret.get("ReplicationStatus", []):
        if regions_to_set_replication:
            for desired_secret_region in regions_to_set_replication:
                if current_secret_region["Region"] == desired_secret_region["region"]:
                    regions_to_set_replication.remove(desired_secret_region)
                else:
                    regions_to_remove_replication.append(current_secret_region["Region"])
        else:
            regions_to_remove_replication.append(current_secret_region["Region"])

    return regions_to_set_replication, regions_to_remove_replication


def main():
    replica_args = dict(
        region=dict(type="str", required=True),
        kms_key_id=dict(type="str", required=False),
    )

    module = AnsibleAWSModule(
        argument_spec={
            "name": dict(required=True),
            "state": dict(choices=["present", "absent"], default="present"),
            "overwrite": dict(type="bool", default=True),
            "description": dict(default=""),
            "replica": dict(type="list", elements="dict", options=replica_args),
            "kms_key_id": dict(),
            "secret_type": dict(choices=["binary", "string"], default="string"),
            "secret": dict(default="", no_log=True),
            "json_secret": dict(type="json", no_log=True),
            "resource_policy": dict(type="json", default=None),
            "tags": dict(type="dict", default=None, aliases=["resource_tags"]),
            "purge_tags": dict(type="bool", default=True),
            "rotation_lambda": dict(),
            "rotation_interval": dict(type="int", default=30),
            "recovery_window": dict(type="int", default=30),
        },
        mutually_exclusive=[["secret", "json_secret"]],
        supports_check_mode=True,
    )

    changed = False
    state = module.params.get("state")
    secrets_mgr = SecretsManagerInterface(module)
    recovery_window = module.params.get("recovery_window")
    secret = Secret(
        module.params.get("name"),
        module.params.get("secret_type"),
        module.params.get("secret") or module.params.get("json_secret"),
        description=module.params.get("description"),
        replica_regions=module.params.get("replica"),
        kms_key_id=module.params.get("kms_key_id"),
        resource_policy=module.params.get("resource_policy"),
        tags=module.params.get("tags"),
        lambda_arn=module.params.get("rotation_lambda"),
        rotation_interval=module.params.get("rotation_interval"),
    )
    purge_tags = module.params.get("purge_tags")

    current_secret = secrets_mgr.get_secret(secret.name)

    if state == "absent":
        if current_secret:
            if not current_secret.get("DeletedDate"):
                result = camel_dict_to_snake_dict(
                    secrets_mgr.delete_secret(secret.name, recovery_window=recovery_window)
                )
                changed = True
            elif current_secret.get("DeletedDate") and recovery_window == 0:
                result = camel_dict_to_snake_dict(
                    secrets_mgr.delete_secret(secret.name, recovery_window=recovery_window)
                )
                changed = True
            else:
                result = "secret already scheduled for deletion"
        else:
            result = "secret does not exist"
    if state == "present":
        if current_secret is None:
            result = secrets_mgr.create_secret(secret)
            if secret.resource_policy and result.get("ARN"):
                result = secrets_mgr.put_resource_policy(secret)
            changed = True
        else:
            # current_secret exists; decide what to do with it
            if current_secret.get("DeletedDate"):
                secrets_mgr.restore_secret(secret.name)
                changed = True
            if not secrets_mgr.secrets_match(secret, current_secret):
                overwrite = module.params.get("overwrite")
                if overwrite:
                    result = secrets_mgr.update_secret(secret)
                    changed = True
            if not rotation_match(secret, current_secret):
                result = secrets_mgr.update_rotation(secret)
                changed = True

            current_resource_policy_response = secrets_mgr.get_resource_policy(secret.name)
            current_resource_policy = current_resource_policy_response.get("ResourcePolicy")
            if compare_policies(secret.resource_policy, current_resource_policy):
                if secret.resource_policy is None and current_resource_policy:
                    result = secrets_mgr.delete_resource_policy(secret.name)
                else:
                    result = secrets_mgr.put_resource_policy(secret)
                changed = True

            if module.params.get("tags") is not None:
                current_tags = boto3_tag_list_to_ansible_dict(current_secret.get("Tags", []))
                tags_to_add, tags_to_remove = compare_aws_tags(current_tags, secret.tags, purge_tags)
                if tags_to_add:
                    secrets_mgr.tag_secret(secret.name, ansible_dict_to_boto3_tag_list(tags_to_add))
                    changed = True
                if tags_to_remove:
                    secrets_mgr.untag_secret(secret.name, tags_to_remove)
                    changed = True

            regions_to_set_replication, regions_to_remove_replication = compare_regions(secret, current_secret)
            if regions_to_set_replication:
                secrets_mgr.replicate_secret(secret.name, regions_to_set_replication)
                changed = True
            if regions_to_remove_replication:
                secrets_mgr.remove_replication(secret.name, regions_to_remove_replication)
                changed = True

        result = camel_dict_to_snake_dict(secrets_mgr.get_secret(secret.name))
        if result.get("tags", None) is not None:
            result["tags_dict"] = boto3_tag_list_to_ansible_dict(result.get("tags", []))
        result.pop("response_metadata")

    module.exit_json(changed=changed, secret=result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

#############################################
#                WARNING                    #
#############################################
#
# This file is auto generated by the
# ansible.content_builder.
#
# Manually editing this file is not advised.
#
# To update the argspec make the desired changes
# in the documentation in the module file and re-run
# ansible.content_builder commenting out
# the path to external 'docstring' in build.yaml.
#
##############################################

"""
The arg spec for the nxos_interfaces module
"""


class InterfacesArgs(object):  # pylint: disable=R0903
    """The arg spec for the nxos_interfaces module"""

    argument_spec = {
        "config": {
            "type": "list",
            "elements": "dict",
            "options": {
                "name": {"type": "str", "required": True},
                "description": {"type": "str"},
                "enabled": {"type": "bool"},
                "speed": {"type": "str"},
                "mode": {"choices": ["layer2", "layer3"], "type": "str"},
                "mtu": {"type": "str"},
                "duplex": {"type": "str", "choices": ["full", "half", "auto"]},
                "ip_forward": {"type": "bool"},
                "fabric_forwarding_anycast_gateway": {"type": "bool"},
                "mac_address": {"type": "str"},
                "logging": {
                    "type": "dict",
                    "options": {
                        "link_status": {"type": "bool"},
                        "trunk_status": {"type": "bool"},
                    },
                },
                "snmp": {
                    "type": "dict",
                    "options": {
                        "trap": {
                            "type": "dict",
                            "options": {"link_status": {"type": "bool"}},
                        },
                    },
                },
                "service_policy": {
                    "type": "dict",
                    "options": {
                        "input": {"type": "str"},
                        "output": {"type": "str"},
                        "type_options": {
                            "type": "dict",
                            "options": {
                                "qos": {
                                    "type": "dict",
                                    "options": {
                                        "input": {"type": "str"},
                                        "output": {"type": "str"},
                                    },
                                },
                                "queuing": {
                                    "type": "dict",
                                    "options": {
                                        "input": {"type": "str"},
                                        "output": {"type": "str"},
                                    },
                                },
                            },
                        },
                    },
                },
            },
        },
        "running_config": {"type": "str"},
        "state": {
            "type": "str",
            "choices": [
                "merged",
                "replaced",
                "overridden",
                "deleted",
                "gathered",
                "rendered",
                "parsed",
                "purged",
            ],
            "default": "merged",
        },
    }  # pylint: disable=C0301



"""Ansible-rulebook event source plugin for Azure Service Bus.

This module provides asynchronous consumers for Azure Service Bus to feed
events into Ansible EDA rulebooks.
"""

import asyncio
import contextlib
import json
import logging
from typing import Any

from azure.servicebus.aio import ServiceBusClient

DOCUMENTATION = r"""
---
short_description: Receive events from an Azure service bus.
description:
  - An ansible-rulebook event source module for receiving events from an Azure service bus.
  - In order to get the service bus and the connection string, refer to
    https://learn.microsoft.com/en-us/azure/service-bus-messaging/service-bus-python-how-to-use-queues?tabs=passwordless
options:
  conn_str:
    description:
      - The connection string to connect to the Azure service bus.
    type: str
    required: true
  queue_name:
    description:
      - The name of the queue to pull messages from.
    type: str
    required: true
  logging_enable:
    description:
      - Whether to turn on logging.
    type: bool
    default: true
  feedback:
    type: bool
    default: false
    description:
      - Should the source plugin wait for feedback before
        processing the next event from the Azure Event Hub
        This flag allows ansible rulebook to pass in an asyncio
        queue which is passed in the args['eda_feedback_queue']
        The source plugin should wait for the response to come
        back on this queue before it picks the next event from
        Azure Event Hub.
  feedback_timeout:
    type: int
    default: 120
    description:
      - Timeout in seconds to wait for feedback from the rule engine
        before raising an exception. Only applies when feedback is enabled.
  eda_feedback_queue:
    description:
      - Provided automatically by ansible-rulebook when the feedback parameter
        is enabled, this parameter utilizes an asyncio.Queue. It allows the
        system to wait for confirmation that an event has been safely persisted
        in the database before removing it from the event bus. Users do not need
        to provide a value for this manually.
"""

EXAMPLES = r"""
- azure.azcollection.azure_service_bus:
    conn_str: "{{connection_str}}"
    queue_name: "{{queue_name}}"
"""

DEFAULT_FEEDBACK_TIMEOUT = 120

logger = logging.getLogger()


async def receive_events(
    queue: asyncio.Queue[Any],
    args: dict[str, Any],  # pylint: disable=W0621
) -> None:
    """Receive events from service bus asynchronously."""
    feedback_enabled = args.get("feedback", False)
    eda_feedback_queue = args.get("eda_feedback_queue")
    feedback_timeout = int(args.get("feedback_timeout", DEFAULT_FEEDBACK_TIMEOUT))

    if feedback_enabled and eda_feedback_queue is None:
        msg = (
            "feedback: true was set but no feedback queue was provided. "
            "This requires a compatible version of ansible-rulebook that "
            "supports the feedback mechanism."
        )
        raise ValueError(msg)
    servicebus_client = ServiceBusClient.from_connection_string(
        conn_str=args["conn_str"],
        logging_enable=bool(args.get("logging_enable", True)),
    )

    async with servicebus_client:
        receiver = servicebus_client.get_queue_receiver(queue_name=args["queue_name"])
        async with receiver:
            async for msg in receiver:
                meta = {"message_id": msg.message_id, "event": {}}
                meta["event"]["uuid"] = msg.message_id
                if msg.enqueued_time_utc:
                    meta["event"]["produced_at"] = msg.enqueued_time_utc.isoformat()

                body = str(msg)
                with contextlib.suppress(json.JSONDecodeError):
                    body = json.loads(body)

                await queue.put({"body": body, "meta": meta})

                if feedback_enabled and eda_feedback_queue:
                    try:
                        await asyncio.wait_for(
                            eda_feedback_queue.get(),
                            timeout=feedback_timeout,
                        )
                        await receiver.complete_message(msg)
                    except asyncio.TimeoutError:
                        logger.exception("Timed out waiting for feedback")
                        raise
                else:
                    await receiver.complete_message(msg)


async def main(
    queue: asyncio.Queue[Any],
    args: dict[str, Any],  # pylint: disable=W0621
) -> None:
    """Receive events from service bus in a loop."""
    await receive_events(queue, args)


if __name__ == "__main__":
    # MockQueue if running directly.

    class MockQueue(asyncio.Queue[Any]):
        """A fake queue."""

        async def put(self: "MockQueue", event: dict[str, Any]) -> None:
            """Print the event."""
            print(event)  # noqa: T201

    args = {
        "conn_str": "Endpoint=sb://foo.servicebus.windows.net/",
        "queue_name": "eda-queue",
    }
    asyncio.run(main(MockQueue(), args))
